@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -26,6 +26,8 @@
 #include <curl/curl.h>
 
 #include "Url.h"
+#include "HtmlFilter.h"
+#include "FilterUtils.h"
 #include "CurlDownloader.h"
 
 using namespace std;
@@ -34,34 +36,42 @@ struct ContentInfo
 {
 	char *m_pContent;
 	size_t m_contentLen;
+	string m_lastModified;
 };
 
-size_t writeCallback(void *pData, size_t dataSize, size_t elementsCount, void *pStream)
+static void freeContentInfo(struct ContentInfo *pInfo)
+{
+	if (pInfo == NULL)
+	{
+		return;
+	}
+
+	if (pInfo->m_pContent != NULL)
+	{
+		free(pInfo->m_pContent);
+		pInfo->m_pContent = NULL;
+		pInfo->m_contentLen = 0;
+	}
+}
+
+static size_t writeCallback(void *pData, size_t dataSize, size_t elementsCount, void *pStream)
 {
 	ContentInfo *pInfo = NULL;
 	size_t totalSize = elementsCount * dataSize;
 
 	if (pStream == NULL)
 	{
-#ifdef DEBUG
-		cout << "writeCallback: no stream" << endl;
-#endif
 		return 0;
 	}
-
 	pInfo = (ContentInfo *)pStream;
+
 	char *pNewContent = (char*)realloc(pInfo->m_pContent, pInfo->m_contentLen + totalSize + 1);
 	if (pNewContent == NULL)
 	{
 #ifdef DEBUG
 		cout << "writeCallback: failed to enlarge buffer" << endl;
 #endif
-		if (pInfo->m_pContent != NULL)
-		{
-			free(pInfo->m_pContent);
-			pInfo->m_pContent = NULL;
-			pInfo->m_contentLen = 0;
-		}
+		freeContentInfo(pInfo);
 		return 0;
 	}
 
@@ -84,6 +94,33 @@ size_t writeCallback(void *pData, size_t dataSize, size_t elementsCount, void *p
 	return totalSize;
 }
 
+static size_t headerCallback(void *pData, size_t dataSize, size_t elementsCount, void *pStream)
+{
+	ContentInfo *pInfo = NULL;
+	size_t totalSize = elementsCount * dataSize;
+
+	if ((pStream == NULL) ||
+		(pData == NULL) ||
+		(totalSize == 0))
+	{
+		return 0;
+	}
+	pInfo = (ContentInfo *)pStream;
+
+	string header((const char*)pData, totalSize);
+
+	string::size_type pos = header.find("Last-Modified: ");
+	if (pos != string::npos)
+	{
+		pInfo->m_lastModified = header.substr(15);
+#ifdef DEBUG
+		cout << "headerCallback: Last-Modified " << pInfo->m_lastModified << endl;
+#endif
+	}
+
+	return totalSize;
+}
+
 unsigned int CurlDownloader::m_initialized = 0;
 
 CurlDownloader::CurlDownloader() :
@@ -157,7 +194,7 @@ Document *CurlDownloader::retrieveUrl(const DocumentInfo &docInfo)
 {
 	Document *pDocument = NULL;
 	string url(Url::escapeUrl(docInfo.getLocation()));
-	long maxRedirectionsCount = 10;
+	unsigned int redirectionsCount = 0;
 
 	if (url.empty() == true)
 	{
@@ -178,7 +215,7 @@ Document *CurlDownloader::retrieveUrl(const DocumentInfo &docInfo)
 
 		curl_easy_setopt(pCurlHandler, CURLOPT_AUTOREFERER, 1);
 		curl_easy_setopt(pCurlHandler, CURLOPT_FOLLOWLOCATION, 1);
-		curl_easy_setopt(pCurlHandler, CURLOPT_MAXREDIRS, maxRedirectionsCount);
+		curl_easy_setopt(pCurlHandler, CURLOPT_MAXREDIRS, 10);
 		curl_easy_setopt(pCurlHandler, CURLOPT_USERAGENT, m_userAgent.c_str());
 		curl_easy_setopt(pCurlHandler, CURLOPT_NOSIGNAL, (long)1);
 		curl_easy_setopt(pCurlHandler, CURLOPT_TIMEOUT, (long)m_timeout);
@@ -187,10 +224,9 @@ Document *CurlDownloader::retrieveUrl(const DocumentInfo &docInfo)
 #endif
 		curl_easy_setopt(pCurlHandler, CURLOPT_WRITEFUNCTION, writeCallback);
 		curl_easy_setopt(pCurlHandler, CURLOPT_WRITEDATA, pContentInfo);
-#ifdef DEBUG
-		cout << "CurlDownloader::retrieveUrl: URL is " << url << endl;
-#endif
-		curl_easy_setopt(pCurlHandler, CURLOPT_URL, url.c_str());
+		curl_easy_setopt(pCurlHandler, CURLOPT_HEADERFUNCTION, headerCallback);
+		curl_easy_setopt(pCurlHandler, CURLOPT_HEADERDATA, pContentInfo);
+
 		// Is a proxy defined ?
 		// Curl automatically checks and makes use of the *_proxy environment variables 
 		if ((m_proxyAddress.empty() == false) &&
@@ -215,10 +251,16 @@ Document *CurlDownloader::retrieveUrl(const DocumentInfo &docInfo)
 			curl_easy_setopt(pCurlHandler, CURLOPT_PROXYTYPE, proxyType);
 		}
 
-		CURLcode res = curl_easy_perform(pCurlHandler);
-		if (res == CURLE_OK)
+#ifdef DEBUG
+		cout << "CurlDownloader::retrieveUrl: URL is " << url << endl;
+#endif
+		while (redirectionsCount < 10)
 		{
-			if ((pContentInfo->m_pContent != NULL) &&
+			curl_easy_setopt(pCurlHandler, CURLOPT_URL, url.c_str());
+
+			CURLcode res = curl_easy_perform(pCurlHandler);
+			if ((res == CURLE_OK) &&
+				(pContentInfo->m_pContent != NULL) &&
 				(pContentInfo->m_contentLen > 0))
 			{
 				char *pContentType = NULL;
@@ -237,16 +279,50 @@ Document *CurlDownloader::retrieveUrl(const DocumentInfo &docInfo)
 					pDocument->setType(pContentType);
 				}
 
+				// The Last-Modified date ?
+				if (pContentInfo->m_lastModified.empty() == false)
+				{
+					pDocument->setTimestamp(pContentInfo->m_lastModified);
+				}
+
+				// Any REFRESH META tag ?
+				Dijon::HtmlFilter htmlFilter("text/html");
+				if ((FilterUtils::feedFilter(*pDocument, &htmlFilter) == true) &&
+					(htmlFilter.next_document() == true))
+				{
+					const map<string, string> &metaData = htmlFilter.get_meta_data();
+					map<string, string>::const_iterator refreshIter = metaData.find("refresh");
+					if (refreshIter != metaData.end())
+					{
+						// Try again
+						string::size_type urlPos = refreshIter->second.find("URL=");
+						if (urlPos != string::npos)
+						{
+							url = refreshIter->second.substr(urlPos + 4);
 #ifdef DEBUG
-				cout << "CurlDownloader::retrieveUrl: document size is " << pContentInfo->m_contentLen << endl;
+							cout << "CurlDownloader::retrieveUrl: redirected to URL " << url << endl;
+#endif
+							delete pDocument;
+							pDocument = NULL;
+							freeContentInfo(pContentInfo);
+							++redirectionsCount;
+							continue;
+						}
+					}
+				}
+#ifdef DEBUG
+				else cout << "CurlDownloader::retrieveUrl: failed to parse HTML" << endl;
 #endif
 			}
+			else
+			{
+				cerr << "Couldn't download " << url << ": " << curl_easy_strerror(res) << endl;
+			}
+
+			break;
 		}
 
-		if (pContentInfo->m_pContent != NULL)
-		{
-			free(pContentInfo->m_pContent);
-		}
+		freeContentInfo(pContentInfo);
 		delete pContentInfo;
 
 		// Cleanup
