@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -118,7 +118,7 @@ bool DownloaderInterface::setSetting(const string &name, const string &value)
 }
 
 /// Sets timeout.
-void DownloaderInterface::setTimeout(unsigned int milliseconds)
+void DownloaderInterface::setTimeout(unsigned int seconds)
 {
-	m_timeout = milliseconds;
+	m_timeout = seconds;
 }
