@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -45,7 +45,7 @@ class DownloaderInterface
 		virtual bool setSetting(const std::string &name, const std::string &value);
 
 		/// Sets timeout.
-		virtual void setTimeout(unsigned int milliseconds);
+		virtual void setTimeout(unsigned int seconds);
 
 		/// Retrieves the specified document; NULL if error. Caller deletes.
 		virtual Document *retrieveUrl(const DocumentInfo &docInfo) = 0;
