@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -50,11 +50,9 @@ Document *FileCollector::retrieveUrl(const DocumentInfo &docInfo)
 		return NULL;
 	}
 
-	string directoryName = thisUrl.getLocation();
-	string fileName = thisUrl.getFile();
-	string fileLocation = directoryName;
+	string fileLocation(thisUrl.getLocation());
 	fileLocation += "/";
-	fileLocation += fileName;
+	fileLocation += thisUrl.getFile();
 
 	Document *pDocument = new Document(docInfo);
 
@@ -71,3 +69,4 @@ Document *FileCollector::retrieveUrl(const DocumentInfo &docInfo)
 
 	return pDocument;
 }
+
