@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -56,6 +56,10 @@ class DBusIndex : public IndexInterface
 		/// Asks the D-Bus service to reload its configuration.
 		static bool reload(void);
 
+		/// Gets some statistics from the D-Bus service.
+		static bool getStatistics(unsigned int crawledCount, unsigned int docsCount,
+			bool &lowDiskSpace, bool &onBattery, bool &crawling);
+
 		/// Returns false if the index couldn't be opened.
 		virtual bool isGood(void) const;
 
