@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -283,3 +283,10 @@ DownloaderInterface *WebEngine::getDownloader(void)
 {
 	return m_pDownloader;
 }
+
+/// Specifies values for editable parameters.
+void WebEngine::setEditableValues(const std::map<std::string, std::string> &editableValues)
+{
+	m_editableValues = editableValues;
+}
+
