@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,6 +21,7 @@
 #include <sys/time.h>
 #include <glib.h>
 #include <map>
+#include <set>
 #include <algorithm>
 #include <iostream>
 #include <utility>
@@ -35,6 +36,7 @@ using std::endl;
 using std::string;
 using std::vector;
 using std::map;
+using std::set;
 using std::find;
 using namespace Dijon;
 
@@ -42,8 +44,8 @@ unsigned int AbstractGenerator::m_maxSeedTerms = 5;
 unsigned int AbstractGenerator::m_minTermPositions = 10;
 
 AbstractGenerator::PositionWindow::PositionWindow() :
-	m_backWeight(0),
-	m_forwardWeight(0)
+	m_backWeight(1),
+	m_forwardWeight(1)
 {
 }
 
@@ -100,12 +102,16 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 #endif
 		try
 		{
+			set<Xapian::termpos> positions;
+
 			// Go through that term's position list in the document
 			for (Xapian::PositionIterator positionIter = m_pIndex->positionlist_begin(docId, termName);
 				positionIter != m_pIndex->positionlist_end(docId, termName); ++positionIter)
 			{
 				Xapian::termpos termPos = *positionIter;
 
+				positions.insert(termPos);
+
 				// Take all the top term's positions into account, and some of 
 				// the other terms' too if the minimum number is not reached
 				if ((m_minTermPositions > abstractWindows.size()) ||
@@ -118,6 +124,12 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 				for (map<Xapian::termpos, PositionWindow>::iterator winIter = abstractWindows.begin();
 					winIter != abstractWindows.end(); ++winIter)
 				{
+					if (positions.find(winIter->first) != positions.end())
+					{
+						// This is for the same term at another position
+						continue;
+					}
+
 					// Is this within the number of words we are interested in ?
 					if (winIter->first > termPos)
 					{
@@ -126,12 +138,9 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 							++winIter->second.m_backWeight;
 						}
 					}
-					else
+					else if (termPos - winIter->first <= m_wordsCount)
 					{
-						if (termPos - winIter->first <= m_wordsCount)
-						{
-							++winIter->second.m_forwardWeight;
-						}
+						++winIter->second.m_forwardWeight;
 					}
 				}
 			}
@@ -158,6 +167,16 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 	for (map<Xapian::termpos, PositionWindow>::iterator winIter = abstractWindows.begin();
 		winIter != abstractWindows.end(); ++winIter)
 	{
+#ifdef DEBUG
+		cout << "AbstractGenerator::generateAbstract: position " << winIter->first
+			<< " weights " << winIter->second.m_backWeight 
+			<< "/" << winIter->second.m_forwardWeight << endl;
+#endif
+		if (bestWeight < winIter->second.m_forwardWeight)
+		{
+			bestPosition = startPosition = winIter->first;
+			bestWeight = winIter->second.m_forwardWeight;
+		}
 		if (bestWeight < winIter->second.m_backWeight)
 		{
 			bestPosition = winIter->first;
@@ -168,11 +187,6 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 			}
 			bestWeight = winIter->second.m_backWeight;
 		}
-		if (bestWeight < winIter->second.m_forwardWeight)
-		{
-			bestPosition = startPosition = winIter->first;
-			bestWeight = winIter->second.m_forwardWeight;
-		}
 	}
 #ifdef DEBUG
 	cout << "AbstractGenerator::generateAbstract: best position is "
@@ -196,6 +210,9 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 			if ((tokenizer.has_cjkv(termName) == true) &&
 				(termName.length() > 4))
 			{
+#ifdef DEBUG
+				cout << "AbstractGenerator::generateAbstract: skipping " << termName << endl;
+#endif
 				continue;
 			}
 
@@ -249,7 +266,7 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 		}
 
 		// Is this a seed term ?
-		if (find(seedTerms.begin(), seedTerms.end(), wordIter->second) != seedTerms.end())
+		if (abstractWindows.find(wordIter->first) != abstractWindows.end())
 		{
 			summary += "<b>";
 			summary += pEscToken;
