@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -24,6 +24,7 @@
 #include <cstring>
 #include <vector>
 #include <iostream>
+#include <fstream>
 #include <algorithm>
 
 #include "config.h"
@@ -49,6 +50,8 @@ using std::cout;
 using std::cerr;
 using std::endl;
 using std::inserter;
+using std::getline;
+using std::ifstream;
 using namespace Dijon;
 
 static void checkFilter(const string &freeQuery, string::size_type filterValueStart,
@@ -88,8 +91,14 @@ static void checkFilter(const string &freeQuery, string::size_type filterValueSt
 class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
 {
 	public:
-		TimeValueRangeProcessor(Xapian::valueno valueNumber) : Xapian::ValueRangeProcessor(), m_valueNumber(valueNumber) { }
-		~TimeValueRangeProcessor() { }
+		TimeValueRangeProcessor(Xapian::valueno valueNumber) :
+			Xapian::ValueRangeProcessor(),
+			m_valueNumber(valueNumber)
+		{
+		}
+		~TimeValueRangeProcessor()
+		{
+		}
 
 		virtual Xapian::valueno operator()(string &begin, string &end)
 		{
@@ -131,8 +140,14 @@ class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
 class PrefixDecider : public Xapian::ExpandDecider
 {
 	public:
-		PrefixDecider(const string &allowedPrefixes) : Xapian::ExpandDecider(), m_allowedPrefixes(allowedPrefixes) { }
-		~PrefixDecider() { }
+		PrefixDecider(const string &allowedPrefixes) :
+			Xapian::ExpandDecider(),
+			m_allowedPrefixes(allowedPrefixes)
+		{
+		}
+		~PrefixDecider()
+		{
+		}
 
 		virtual bool operator()(const std::string &term) const
 		{
@@ -153,17 +168,100 @@ class PrefixDecider : public Xapian::ExpandDecider
 
 };
 
+class FileStopper : public Xapian::SimpleStopper
+{
+	public:
+		FileStopper(const string &languageCode) :
+			Xapian::SimpleStopper(),
+			m_languageCode(languageCode),
+			m_stopwordsCount(0)
+		{
+			if (languageCode.empty() == false)
+			{
+				ifstream inputFile;
+				string fileName(PREFIX);
+
+				fileName += "/share/pinot/stopwords/stopwords.";
+				fileName += languageCode;
+				inputFile.open(fileName.c_str());
+				if (inputFile.good() == true)
+				{
+					string line;
+
+					// Each line is a stopword
+					while (getline(inputFile, line).eof() == false)
+					{
+						add(line);
+						++m_stopwordsCount;
+					}
+				}
+				inputFile.close();
+
+#ifdef DEBUG
+				cout << "FileStopper: " << m_stopwordsCount << " stopwords for language code " << languageCode << endl;
+#endif
+			}
+		}
+		virtual ~FileStopper()
+		{
+		}
+
+		unsigned int get_stopwords_count(void) const
+		{
+			return m_stopwordsCount;
+		}
+
+		static FileStopper *get_stopper(const string &languageCode)
+		{
+			if (m_pStopper == NULL)
+			{
+				m_pStopper = new FileStopper(languageCode);
+			}
+			else if (m_pStopper->m_languageCode != languageCode)
+			{
+				delete m_pStopper;
+
+				m_pStopper = new FileStopper(languageCode);
+			}
+
+			return m_pStopper;
+		}
+
+		static void free_stopper(void)
+		{
+			if (m_pStopper != NULL)
+			{
+				delete m_pStopper;
+				m_pStopper = NULL;
+			}
+		}
+
+	protected:
+		string m_languageCode;
+		unsigned int m_stopwordsCount;
+		static FileStopper *m_pStopper;
+
+};
+
+FileStopper *FileStopper::m_pStopper = NULL;
+
 class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 {
 	public:
 		typedef enum { NONE = 0, BRACKETS } CJKVWrap;
 
-		QueryModifier(const string &query, unsigned int nGramSize) :
+		QueryModifier(const string &query,
+			bool diacriticSensitive, unsigned int nGramSize) :
 			m_query(query),
+			m_diacriticSensitive(diacriticSensitive),
 			m_pos(0),
 			m_wrap(BRACKETS),
+			m_wrapped(false),
 			m_nGramCount(0),
-			m_nGramSize(nGramSize)
+			m_nGramSize(nGramSize),
+			m_tokensCount(0),
+			m_hasCJKV(false),
+			m_hasNonCJKV(false)
 		{
 		}
 
@@ -183,6 +281,7 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 
 			// Where is this token in the original query ?
 			string::size_type tokPos = m_query.find(tok, m_pos);
+			++m_tokensCount;
 
 			// Is this CJKV ?
 			if (is_cjkv == false)
@@ -214,26 +313,32 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 					// It's a filter
 					m_wrap = NONE;
 					m_currentFilter = tok;
-#ifdef DEBUG
-					cout << "QueryModifier::handle_token: in filter" << endl;
-#endif
 				}
 				else
 				{
-#ifdef DEBUG
-					cout << "QueryModifier::handle_token: in brackets" << endl;
-#endif
 					m_wrap = BRACKETS;
 				}
 
+				if (m_currentFilter.empty() == true)
+				{
+					m_hasNonCJKV = true;
+				}
+
+				if (m_diacriticSensitive == false)
+				{
+					// Strip accents and other diacritics from terms
+					string unaccentedTok(StringManip::stripDiacritics(tok));
+					if (tok != unaccentedTok)
+					{
+						m_query.replace(tokPos, tok.length(), unaccentedTok);
+					}
+				}
+
 				// Return right away
 				return true;
 			}
 
 			// First n-gram ?
-#ifdef DEBUG
-			cout << "QueryModifier::handle_token: n-gram " << m_nGramCount << endl;
-#endif
 			if (m_nGramCount == 0)
 			{
 				if (tokPos == string::npos)
@@ -248,15 +353,8 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 					m_modifiedQuery += " " + m_query.substr(m_pos, tokPos - m_pos);
 				}
 				m_pos += tok.length();
-				switch (m_wrap)
-				{
-					case BRACKETS:
-						m_modifiedQuery += " (";
-						break;
-					case NONE:
-					default:
-						break;
-				}
+
+				wrapOpen();
 			}
 			else
 			{
@@ -271,44 +369,85 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 			cout << "QueryModifier::handle_token: " << m_modifiedQuery << endl;
 #endif
 
-			++m_nGramCount;
-			if ((m_nGramCount % m_nGramSize == 0) &&
-				(tokPos != string::npos))
+			if (tokPos != string::npos)
 			{
 				m_pos = tokPos + tok.length();
 			}
+			++m_nGramCount;
+			m_hasCJKV = true;
 
 			return true;
 		}
 
-		string get_modified_query(void)
+		unsigned int get_tokens_count(void) const
+		{
+			return m_tokensCount;
+		}
+
+		string get_modified_query(bool &pureCJKV)
 		{
 #ifdef DEBUG
 			cout << "QueryModifier::get_modified_query: " << m_pos << "/" << m_query.length() << endl;
 #endif
-			if (m_pos + 1 < m_query.length())
+
+			// Anything left ?
+			if (m_pos < m_query.length() - 1)
 			{
-				m_modifiedQuery += " " + m_query.substr(m_pos + 1);
+				m_modifiedQuery += " " + m_query.substr(m_pos);
 			}
 			wrapClose();
 #ifdef DEBUG
 			cout << "QueryModifier::get_modified_query: " << m_modifiedQuery << endl;
 #endif
 
+			if ((m_hasCJKV == true) &&
+				(m_hasNonCJKV == false))
+			{
+				pureCJKV = true;
+			}
+			else
+			{
+				pureCJKV = false;
+			}
+
 			return m_modifiedQuery;
 		}
 
 	protected:
 		string m_query;
+		bool m_diacriticSensitive;
 		string m_modifiedQuery;
 		string::size_type m_pos;
 		CJKVWrap m_wrap;
+		bool m_wrapped;
 		string m_currentFilter;
 		unsigned int m_nGramCount;
 		unsigned int m_nGramSize;
+		unsigned int m_tokensCount;
+		bool m_hasCJKV;
+		bool m_hasNonCJKV;
+
+		void wrapOpen(void)
+		{
+			switch (m_wrap)
+			{
+				case BRACKETS:
+					m_modifiedQuery += " (";
+					break;
+				case NONE:
+				default:
+					break;
+			}
+			m_wrapped = true;
+		}
 
 		void wrapClose(void)
 		{
+			if (m_wrapped == false)
+			{
+				return;
+			}
+
 			// Finish wrapping CJKV tokens
 			switch (m_wrap)
 			{
@@ -319,6 +458,7 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 				default:
 					break;
 			}
+			m_wrapped = false;
 		}
 
 };
@@ -348,26 +488,50 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	Xapian::QueryParser parser;
 	Xapian::Stem stemmer;
 	CJKVTokenizer tokenizer;
-	string freeQuery(StringManip::replaceSubString(queryProps.getFreeQuery(), "\n", " "));
-	unsigned int minDay, minMonth, minYear = 0;
-	unsigned int maxDay, maxMonth, maxYear = 0;
+	string freeQuery(queryProps.getFreeQuery());
+	unsigned int tokensCount = 1;
+	bool diacriticSensitive = queryProps.getDiacriticSensitive();
 
-	if (tokenizer.has_cjkv(freeQuery) == true)
+	// Modifying the query is necessary if it's CJKV or diacritics are off
+	if ((tokenizer.has_cjkv(freeQuery) == true) ||
+		(diacriticSensitive == false))
 	{
-		QueryModifier handler(freeQuery, tokenizer.get_ngram_size());
+		QueryModifier handler(freeQuery,
+			diacriticSensitive,
+			tokenizer.get_ngram_size());
 
 		tokenizer.tokenize(freeQuery, handler, true);
 
-		string cjkvQuery(handler.get_modified_query());
+		tokensCount = handler.get_tokens_count();
+
+		// We can disable stemming and spelling correction for pure CJKV queries
+		string cjkvQuery(handler.get_modified_query(minimal));
 #ifdef DEBUG
 		cout << "XapianEngine::parseQuery: CJKV query is " << cjkvQuery << endl;
 #endif
 
 		// Do as if the user had given this as input
 		freeQuery = cjkvQuery;
-		// We can disable stemming and spelling correction
-		minimal = true;
 	}
+	else
+	{
+		string::size_type spacePos = freeQuery.find(' ');
+		while (spacePos != string::npos)
+		{
+			++tokensCount;
+
+			if (spacePos + 1 >= freeQuery.length())
+			{
+				break;
+			}
+
+			// Next
+			spacePos = freeQuery.find(' ', spacePos + 1);
+		}
+	}
+#ifdef DEBUG
+	cout << "XapianEngine::parseQuery: " << tokensCount << " tokens" << endl;
+#endif
 
 	if (pIndex != NULL)
 	{
@@ -392,6 +556,19 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 		}
 		parser.set_stemmer(stemmer);
 		parser.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
+
+		// Don't bother loading the stopwords list if there's only one token
+		if (tokensCount > 1)
+		{
+			string languageCode(Languages::toCode(stemLanguage));
+
+			FileStopper *pStopper = FileStopper::get_stopper(languageCode);
+			if ((pStopper != NULL) &&
+				(pStopper->get_stopwords_count() > 0))
+			{
+				parser.set_stopper(pStopper);
+			}
+		}
 	}
 	else
 	{
@@ -465,6 +642,7 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 
 		XapianQueryBuilder builder(parser, fieldMapping);
 		XesamParser *pParser = NULL;
+		bool parsedQuery = false;
 
 		// Get a Xesam parser
 		if (type == QueryProperties::XESAM_QL)
@@ -480,14 +658,14 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 
 		if (pParser != NULL)
 		{
-			bool parsedQuery = pParser->parse(freeQuery, builder);
+			parsedQuery = pParser->parse(freeQuery, builder);
 
 			delete pParser;
+		}
 
-			if (parsedQuery == true)
-			{
-				return builder.get_query();
-			}
+		if (parsedQuery == true)
+		{
+			return builder.get_query();
 		}
 
 		return Xapian::Query();
@@ -549,8 +727,7 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 
 	// Parse the query string with all necessary options
 	unsigned int flags = Xapian::QueryParser::FLAG_BOOLEAN|Xapian::QueryParser::FLAG_PHRASE|
-		Xapian::QueryParser::FLAG_LOVEHATE|Xapian::QueryParser::FLAG_BOOLEAN_ANY_CASE|
-		Xapian::QueryParser::FLAG_PURE_NOT;
+		Xapian::QueryParser::FLAG_LOVEHATE|Xapian::QueryParser::FLAG_PURE_NOT;
 	if (minimal == false)
 	{
 		flags |= Xapian::QueryParser::FLAG_WILDCARD;
@@ -747,6 +924,7 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 			// Get 10 non-prefixed terms
 			string allowedPrefixes("RS");
 			PrefixDecider expandDecider(allowedPrefixes);
+			CJKVTokenizer tokenizer;
 			Xapian::ESet expandTerms = enquire.get_eset(20, expandDocs, &expandDecider);
 #ifdef DEBUG
 			cout << "XapianEngine::queryDatabase: " << expandTerms.size() << " expand terms" << endl;
@@ -754,16 +932,23 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 			for (Xapian::ESetIterator termIter = expandTerms.begin();
 				(termIter != expandTerms.end()) && (count < 10); ++termIter)
 			{
-				char firstChar = (*termIter)[0];
+				string expandTerm(*termIter);
+				char firstChar = expandTerm[0];
 
+				// Skip short terms
+				if ((tokenizer.has_cjkv(expandTerm) == false) &&
+					(expandTerm.length() < 3))
+				{
+					continue;
+				}
+
+				// Is this prefixed ?
 				if (allowedPrefixes.find(firstChar) != string::npos)
 				{
-					m_expandTerms.insert((*termIter).substr(1));
-				}
-				else
-				{
-					m_expandTerms.insert(*termIter);
+					expandTerm.erase(0, 1);
 				}
+
+				m_expandTerms.insert(expandTerm);
 				++count;
 			}
 		}
@@ -783,6 +968,12 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 	return false;
 }
 
+/// Frees all objects.
+void XapianEngine::freeAll(void)
+{
+	FileStopper::free_stopper();
+}
+
 //
 // Implementation of SearchEngineInterface
 //
