@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -55,39 +55,6 @@ using std::map;
 using std::min;
 using std::max;
 
-static bool setVersionFile(const string &databaseName, const string &version)
-{
-	ofstream verFile;
-	string verFileName(databaseName + "/version");
-	bool setVer = false;
-
-	verFile.open(verFileName.c_str(), ios::trunc);
-	if (verFile.good() == true)
-	{
-		verFile << version;
-		setVer = true;
-	}
-	verFile.close();
-
-	return setVer;
-}
-
-static string getVersionFromFile(const string &databaseName)
-{
-	ifstream verFile;
-	string verFileName(databaseName + "/version");
-	string version;
-
-	verFile.open(verFileName.c_str());
-	if (verFile.good() == true)
-	{
-		verFile >> version;
-	}
-	verFile.close();
-
-	return version;
-}
-
 class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 {
 	public:
@@ -103,12 +70,18 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 			m_nGramSize(nGramSize),
 			m_nGramCount(0),
 			m_doSpelling(doSpelling),
-			m_termPos(termPos)
+			m_termPos(termPos),
+			m_hasCJKV(false)
 		{
 		}
 
 		virtual ~TokensIndexer()
 		{
+			if (m_hasCJKV == true)
+			{
+				// This will help identify CJKV documents
+				m_doc.add_term("XTOK:CJKV");
+			}
 		}
 
 		virtual bool handle_token(const string &tok, bool is_cjkv)
@@ -134,6 +107,18 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 			// Is this CJKV ?
 			if (is_cjkv == false)
 			{
+#ifndef _DIACRITICS_SENSITIVE
+				bool hasDiacritics = false;
+
+				// Remove accents and other diacritics
+				string unaccentedTerm(StringManip::stripDiacritics(term));
+				if (unaccentedTerm != term)
+				{
+					m_doc.add_posting(m_prefix + XapianDatabase::limitTermLength(unaccentedTerm), m_termPos);
+					hasDiacritics = true;
+				}
+#endif
+
 				// Don't stem if the term starts with a digit
 				if ((m_pStemmer != NULL) &&
 					(isdigit((int)term[0]) == 0))
@@ -141,6 +126,14 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 					string stemmedTerm((*m_pStemmer)(term));
 
 					m_doc.add_term("Z" + XapianDatabase::limitTermLength(stemmedTerm));
+#ifndef _DIACRITICS_SENSITIVE
+					if (hasDiacritics == true)
+					{
+						stemmedTerm = (*m_pStemmer)(unaccentedTerm);
+
+						m_doc.add_term("Z" + XapianDatabase::limitTermLength(stemmedTerm));
+					}
+#endif
 				}
 
 				addSpelling = m_doSpelling;
@@ -158,6 +151,7 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 					addSpelling = m_doSpelling;
 				}
 				++m_nGramCount;
+				m_hasCJKV = true;
 			}
 
 			if (addSpelling == true)
@@ -186,6 +180,7 @@ class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 		unsigned int m_nGramCount;
 		bool &m_doSpelling;
 		Xapian::termcount &m_termPos;
+		bool m_hasCJKV;
 
 };
 
@@ -310,15 +305,20 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 		Dijon::CJKVTokenizer tokenizer;
 		string text(pRawData);
 
+#ifdef _DIACRITICS_SENSITIVE
 		if (tokenizer.has_cjkv(text) == true)
 		{
+#endif
 			// Use overload
 			addPostingsToDocument(tokenizer, pStemmer, text, doc, db,
 				prefix, doSpelling, termPos);
 			isCJKV = true;
+#ifdef _DIACRITICS_SENSITIVE
 		}
+#endif
 	}
 
+#ifdef _DIACRITICS_SENSITIVE
 	if (isCJKV == false)
 	{
 		Xapian::TermGenerator generator;
@@ -359,6 +359,7 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 		}
 		termPos = generator.get_termpos();
 	}
+#endif
 
 	if (pStemmer != NULL)
 	{
@@ -375,13 +376,9 @@ void XapianIndex::addPostingsToDocument(Dijon::CJKVTokenizer &tokenizer, Xapian:
 
 	// Get the terms
 	tokenizer.tokenize(text, handler);
-
 #ifdef DEBUG
-	cout << "XapianIndex::addPostingsToDocument: CJKV terms to position " << termPos << endl;
+	cout << "XapianIndex::addPostingsToDocument: terms to position " << termPos << endl;
 #endif
-
-	// This will help identify which documents were processed here
-	doc.add_term("XTOK:CJKV");
 }
 
 void XapianIndex::addLabelsToDocument(Xapian::Document &doc, const set<string> &labels,
@@ -927,7 +924,7 @@ string XapianIndex::getMetadata(const string &name) const
 	if (pDatabase == NULL)
 	{
 		cerr << "Bad index " << m_databaseName << endl;
-		return false;
+		return "";
 	}
 
 	try
@@ -1178,17 +1175,7 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 /// Sets the list of known labels.
 bool XapianIndex::setLabels(const set<string> &labels)
 {
-	bool setLabels = false;
-
-#if ENABLE_XAPIAN_DB_METADATA>0
-	string labelString;
-
-	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
-	if (pDatabase == NULL)
-	{
-		cerr << "Bad index " << m_databaseName << endl;
-		return false;
-	}
+	string labelsString;
 
 	for (set<string>::const_iterator labelIter = labels.begin();
 		labelIter != labels.end(); ++labelIter)
@@ -1198,89 +1185,40 @@ bool XapianIndex::setLabels(const set<string> &labels)
 		{
 			continue;
 		}
-		labelString += "[";
-		labelString += Url::escapeUrl(*labelIter);
-		labelString += "]";
+
+		labelsString += "[";
+		labelsString += Url::escapeUrl(*labelIter);
+		labelsString += "]";
 	}
 
-	try
-	{
-#ifdef DEBUG
-		cout << "XapianIndex::setLabels: " << labels.size() << " labels" << endl;
-#endif
-		Xapian::WritableDatabase *pIndex = pDatabase->writeLock();
-		if (pIndex != NULL)
-		{
-			pIndex->set_metadata("labels", labelString);
-			setLabels = true;
-		}
-	}
-	catch (const Xapian::Error &error)
-	{
-		cerr << "Couldn't set database labels: " << error.get_type() << ": " << error.get_msg() << endl;
-	}
-	catch (...)
-	{
-		cerr << "Couldn't set database labels, unknown exception occured" << endl;
-	}
-	pDatabase->unlock();
-#endif
-
-	return setLabels;
+	return setMetadata("labels", labelsString);
 }
 
 /// Gets the list of known labels.
 bool XapianIndex::getLabels(set<string> &labels) const
 {
-#if ENABLE_XAPIAN_DB_METADATA>0
-	string labelsString;
+	string labelsString(getMetadata("labels"));
 
-	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
-	if (pDatabase == NULL)
+	if (labelsString.empty() == true)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
 		return false;
 	}
 
-	try
+	string::size_type endPos = 0;
+	string label(StringManip::extractField(labelsString, "[", "]", endPos));
+
+	while (label.empty() == false)
 	{
-		Xapian::Database *pIndex = pDatabase->readLock();
-		if (pIndex != NULL)
+		labels.insert(Url::unescapeUrl(label));
+
+		if (endPos == string::npos)
 		{
-			labelsString = pIndex->get_metadata("labels");
+			break;
 		}
+		label = StringManip::extractField(labelsString, "[", "]", endPos);
 	}
-	catch (const Xapian::Error &error)
-	{
-		cerr << "Couldn't get database labels: " << error.get_type() << ": " << error.get_msg() << endl;
-	}
-	catch (...)
-	{
-		cerr << "Couldn't get database labels, unknown exception occured" << endl;
-	}
-	pDatabase->unlock();
 
-	if (labelsString.empty() == false)
-	{
-		string::size_type endPos = 0;
-		string label(StringManip::extractField(labelsString, "[", "]", endPos));
-
-		while (label.empty() == false)
-		{
-			labels.insert(Url::unescapeUrl(label));
-
-			if (endPos == string::npos)
-			{
-				break;
-			}
-			label = StringManip::extractField(labelsString, "[", "]", endPos);
-		}
-
-		return true;
-	}
-#endif
-
-	return false;
+	return true;
 }
 
 /// Adds a label.
@@ -1632,6 +1570,14 @@ unsigned int XapianIndex::hasDocument(const string &url) const
 /// Gets terms with the same root.
 unsigned int XapianIndex::getCloseTerms(const string &term, set<string> &suggestions)
 {
+	Dijon::CJKVTokenizer tokenizer;
+
+	// Only offer suggestions for non CJKV terms
+	if (tokenizer.has_cjkv(term) == true)
+	{
+		return 0;
+	}
+
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
@@ -1658,12 +1604,9 @@ unsigned int XapianIndex::getCloseTerms(const string &term, set<string> &suggest
 				{
 					string suggestedTerm(*termIter);
 
+					// Does this term have the same root ?
 					if (suggestedTerm.find(baseTerm) != 0)
 					{
-						// This term doesn't have the same root
-#ifdef DEBUG
-						cout << "XapianIndex::getCloseTerms: not the same root" << endl;
-#endif
 						break;
 					}
 
