@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -51,12 +51,11 @@ static struct option g_longOptions[] = {
 
 static void printHelp(void)
 {
-	map<string, bool> engines;
+	map<ModuleProperties, bool> engines;
 
 	// Help
 	ModuleFactory::loadModules(string(LIBDIR) + string("/pinot/backends"));
 	ModuleFactory::getSupportedEngines(engines);
-	ModuleFactory::unloadModules();
 	cout << "pinot-index - Index documents from the command-line\n\n"
 		<< "Usage: pinot-index [OPTIONS] URLS\n\n"
 		<< "Options:\n"
@@ -71,13 +70,15 @@ static void printHelp(void)
 		<< "  -s, --showinfo            show information about the document\n"
 		<< "  -v, --version             output version information and exit\n\n"
 		<< "Supported back-ends are :";
-	for (map<string, bool>::const_iterator engineIter = engines.begin(); engineIter != engines.end(); ++engineIter)
+	for (map<ModuleProperties, bool>::const_iterator engineIter = engines.begin(); engineIter != engines.end(); ++engineIter)
 	{
-		if (engineIter->second == true)
+		if ((engineIter->second == true) &&
+			(ModuleFactory::isSupported(engineIter->first.m_name, true) == true))
 		{
-			cout << " " << engineIter->first;
+			cout << " '" << engineIter->first.m_name << "'";
 		}
 	}
+	ModuleFactory::unloadModules();
 	cout << "\n\nExamples:\n"
 		<< "pinot-index --check --showinfo --backend xapian --db ~/.pinot/daemon file:///home/fabrice/Documents/Bozo.txt\n\n"
 		<< "pinot-index --index --db ~/.pinot/index http://pinot.berlios.de/\n\n"
@@ -352,6 +353,14 @@ int main(int argc, char **argv)
 				}
 				cout << endl;
 			}
+
+			vector<MIMEAction> typeActions;
+			MIMEScanner::getDefaultActions(docInfo.getType(), typeActions);
+			for (vector<MIMEAction>::const_iterator actionIter = typeActions.begin();
+				actionIter != typeActions.end(); ++actionIter)
+			{
+				cout << "Action   : '" << actionIter->m_name << "' " << actionIter->m_exec << endl;
+			}
 		}
 
 		// Next
