@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006,2007 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -40,6 +40,7 @@ static struct option g_longOptions[] = {
 	{"proxyaddress", 1, 0, 'a'},
 	{"proxyport", 1, 0, 'p'},
 	{"proxytype", 1, 0, 't'},
+	{"seteditable", 1, 0, 'e'},
 	{"stemming", 1, 0, 's'},
 	{"tocsv", 1, 0, 'c'},
 	{"toxml", 1, 0, 'x'},
@@ -79,7 +80,7 @@ static bool loadFile(const string &xesamFile, string &fileContents)
 
 static void printHelp(void)
 {
-	map<string, bool> engines;
+	map<ModuleProperties, bool> engines;
 
 	// Help
 	ModuleFactory::loadModules(string(LIBDIR) + string("/pinot/backends"));
@@ -94,37 +95,38 @@ static void printHelp(void)
 		<< "  -p, --proxyport           proxy port\n"
 		<< "  -t, --proxytype           proxy type (default HTTP, SOCKS4, SOCKS5)\n"
 		<< "  -s, --stemming            stemming language (in English)\n"
+		<< "  -e, --seteditable         plugin editable parameter, name:value pair\n"
 		<< "  -c, --tocsv               file to export results in CSV format to\n"
 		<< "  -x, --toxml               file to export results in XML format to\n"
 		<< "  -v, --version             output version information and exit\n"
 		<< "  -q, --xesamql             query input is a file containing Xesam QL\n"
 		<< "  -u, --xesamul             query input is a file containing Xesam UL\n\n"
 		<< "Supported search engine types are :";
-	for (map<string, bool>::const_iterator engineIter = engines.begin(); engineIter != engines.end(); ++engineIter)
+	for (map<ModuleProperties, bool>::const_iterator engineIter = engines.begin(); engineIter != engines.end(); ++engineIter)
 	{
-		cout << " " << engineIter->first;
+		cout << " '" << engineIter->first.m_name << "'";
 	}
 	cout << "\n\nExamples:\n"
-#ifdef HAVE_GOOGLEAPI
-		<< "pinot-search googleapi mygoogleapikey \"clowns\"\n\n"
-#endif
 		<< "pinot-search opensearch " << PREFIX << "/share/pinot/engines/KrustyDescription.xml \"clowns\"\n\n"
-		<< "pinot-search --max 20 sherlock " << PREFIX << "/share/pinot/engines/Bozo.src \"clowns\"\n\n"
+		<< "pinot-search --max 20 sherlock --seteditable \"Bozo App ID:1234567890\" " << PREFIX << "/share/pinot/engines/Bozo.src \"clowns\"\n\n"
+		<< "pinot-search googleapi mygoogleapikey \"clowns\"\n\n"
 		<< "pinot-search xapian ~/.pinot/index \"label:Clowns\"\n\n"
 		<< "pinot-search --stemming english xapian somehostname:12345 \"clowning\"\n\n"
+		<< "pinot-search --xesamul xapian ~/.pinot/index some_xesamul_query.txt\n\n"
+		<< "pinot-search --xesamql xesam - some_xesamql_query.xml\n\n"
 		<< "Report bugs to " << PACKAGE_BUGREPORT << endl;
 }
 
 int main(int argc, char **argv)
 {
 	QueryProperties::QueryType queryType = QueryProperties::XAPIAN_QP;
-	string engineType, option, csvExport, xmlExport, proxyAddress, proxyPort, proxyType, stemLanguage;
+	string engineType, option, csvExport, xmlExport, proxyAddress, proxyPort, proxyType, editableParameter, stemLanguage;
 	unsigned int maxResultsCount = 10; 
 	int longOptionIndex = 0;
 	bool printResults = true;
 
 	// Look at the options
-	int optionChar = getopt_long(argc, argv, "c:hm:a:p:qs:t:uvx:", g_longOptions, &longOptionIndex);
+	int optionChar = getopt_long(argc, argv, "c:hm:a:e:p:qs:t:uvx:", g_longOptions, &longOptionIndex);
 	while (optionChar != -1)
 	{
 		switch (optionChar)
@@ -135,6 +137,12 @@ int main(int argc, char **argv)
 					proxyAddress = optarg;
 				}
 				break;
+			case 'e':
+				if (optarg != NULL)
+				{
+					editableParameter = optarg;
+				}
+				break;
 			case 'c':
 				if (optarg != NULL)
 				{
@@ -193,7 +201,7 @@ int main(int argc, char **argv)
 		}
 
 		// Next option
-		optionChar = getopt_long(argc, argv, "c:hm:a:p:qs:t:uvx:", g_longOptions, &longOptionIndex);
+		optionChar = getopt_long(argc, argv, "c:hm:a:e:p:qs:t:uvx:", g_longOptions, &longOptionIndex);
 	}
 
 	if (argc == 1)
@@ -260,6 +268,15 @@ int main(int argc, char **argv)
 			pDownloader->setSetting("proxyport", proxyPort);
 			pDownloader->setSetting("proxytype", proxyType);
 		}
+
+		string::size_type colonPos = editableParameter.find(':');
+		if (colonPos != string::npos)
+		{
+			map<string, string> editableValues;
+
+			editableValues[editableParameter.substr(0, colonPos)] = editableParameter.substr(colonPos + 1);
+			pWebEngine->setEditableValues(editableValues);
+		}
 	}
 
 	// Set the query
