@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -326,14 +326,28 @@ bool CrawlHistory::updateItems(const map<string, time_t> urls, CrawlStatus statu
 }
 
 /// Updates the status of items en masse.
-bool CrawlHistory::updateItemsStatus(unsigned int sourceId, CrawlStatus currentStatus, CrawlStatus newStatus)
+bool CrawlHistory::updateItemsStatus(CrawlStatus currentStatus, CrawlStatus newStatus,
+	unsigned int sourceId, bool allSources)
 {
+	SQLResults *results = NULL;
 	bool success = false;
 
-	SQLResults *results = executeStatement("UPDATE CrawlHistory \
-		SET Status='%q' WHERE SourceId='%u' AND Status='%q';",
-		statusToText(newStatus).c_str(), sourceId,
-		statusToText(currentStatus).c_str());
+	if (allSources == false)
+	{
+		results = executeStatement("UPDATE CrawlHistory \
+			SET Status='%q' WHERE SourceId='%u' AND Status='%q';",
+			statusToText(newStatus).c_str(), sourceId,
+			statusToText(currentStatus).c_str());
+	}
+	else
+	{
+		// Ignore the source
+		results = executeStatement("UPDATE CrawlHistory \
+			SET Status='%q' WHERE Status='%q';",
+			statusToText(newStatus).c_str(),
+			statusToText(currentStatus).c_str());
+	}
+
 	if (results != NULL)
 	{
 		success = true;
@@ -367,6 +381,35 @@ int CrawlHistory::getErrorDetails(const string &url, time_t &date)
 	return errNum;
 }
 
+/// Returns items.
+unsigned int CrawlHistory::getItems(CrawlStatus status, set<string> &urls)
+{
+	unsigned int count = 0;
+
+	SQLResults *results = executeStatement("SELECT Url FROM CrawlHistory \
+		WHERE Status='%q';", statusToText(status).c_str());
+	if (results != NULL)
+	{
+		while (results->hasMoreRows() == true)
+		{
+			SQLRow *row = results->nextRow();
+			if (row == NULL)
+			{
+				break;
+			}
+
+			urls.insert(Url::unescapeUrl(row->getColumn(0)));
+			++count;
+
+			delete row;
+		}
+
+		delete results;
+	}
+
+	return count;
+}
+
 /// Returns items that belong to a source.
 unsigned int CrawlHistory::getSourceItems(unsigned int sourceId, CrawlStatus status,
 	set<string> &urls, time_t minDate)
