@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -64,11 +64,15 @@ class CrawlHistory : public SQLiteBase
 		bool updateItems(const std::map<std::string, time_t> urls, CrawlStatus status);
 
 		/// Updates the status of items en masse.
-		bool updateItemsStatus(unsigned int sourceId, CrawlStatus currentStatus, CrawlStatus newStatus);
+		bool updateItemsStatus(CrawlStatus currentStatus, CrawlStatus newStatus,
+			unsigned int sourceId, bool allSources = false);
 
 		/// Gets the error number and date for a URL.
 		int getErrorDetails(const std::string &url, time_t &date);
 
+		/// Returns items.
+		unsigned int getItems(CrawlStatus status, std::set<std::string> &urls);
+
 		/// Returns items that belong to a source.
 		unsigned int getSourceItems(unsigned int sourceId, CrawlStatus status,
 			std::set<std::string> &urls, time_t minDate = 0);
