@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007,2008 Fabrice Colin
+ *  Copyright 2007-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -38,6 +38,7 @@ using std::string;
 using std::set;
 using std::map;
 
+set<string> FilterUtils::m_types;
 map<string, string> FilterUtils::m_typeAliases;
 
 FilterUtils::FilterUtils()
@@ -79,8 +80,13 @@ Dijon::Filter *FilterUtils::getFilter(const string &mimeType)
 	{
 		set<string> parentTypes;
 
+		if (m_types.empty() == true)
+		{
+			Dijon::FilterFactory::getSupportedTypes(m_types);
+		}
+
 		// Try that type's parents
-		MIMEScanner::getParentTypes(mimeType, parentTypes);
+		MIMEScanner::getParentTypes(mimeType, m_types, parentTypes);
 		for (set<string>::const_iterator parentIter = parentTypes.begin();
 			parentIter != parentTypes.end(); ++parentIter)
 		{
@@ -92,6 +98,9 @@ Dijon::Filter *FilterUtils::getFilter(const string &mimeType)
 				return pFilter;
 			}
 		}
+#ifdef DEBUG
+		cout << "FilterUtils::getFilter: no valid parent for " << mimeType << endl;
+#endif
 
 		// This type has no valid parent
 		m_typeAliases[mimeType] = UNSUPPORTED_TYPE;
@@ -120,8 +129,13 @@ bool FilterUtils::isSupportedType(const string &mimeType)
 	{
 		set<string> parentTypes;
 
+		if (m_types.empty() == true)
+		{
+			Dijon::FilterFactory::getSupportedTypes(m_types);
+		}
+
 		// Try that type's parents
-		MIMEScanner::getParentTypes(mimeType, parentTypes);
+		MIMEScanner::getParentTypes(mimeType, m_types, parentTypes);
 		for (set<string>::const_iterator parentIter = parentTypes.begin();
 			parentIter != parentTypes.end(); ++parentIter)
 		{
@@ -132,6 +146,9 @@ bool FilterUtils::isSupportedType(const string &mimeType)
 				return true;
 			}
 		}
+#ifdef DEBUG
+		cout << "FilterUtils::isSupportedType: no valid parent for " << mimeType << endl;
+#endif
 
 		// This type has no valid parent
 		m_typeAliases[mimeType] = UNSUPPORTED_TYPE;
@@ -248,6 +265,7 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 {
 	string charset, uri, ipath;
+	bool checkType = false;
 
 	if (pFilter == NULL)
 	{
@@ -277,7 +295,16 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 		}
 		else if (metaIter->first == "mimetype")
 		{
-			doc.setType(StringManip::toLowerCase(metaIter->second));
+			string mimeType(StringManip::toLowerCase(metaIter->second));
+
+			if (mimeType == "scan")
+			{
+				checkType = true;
+			}
+			else
+			{
+				doc.setType(StringManip::toLowerCase(metaIter->second));
+			}
 		}
 		else if (metaIter->first == "size")
 		{
@@ -330,6 +357,12 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 			cerr << doc.getLocation() << " may not have been fully converted to UTF-8" << endl;
 		}
 		doc.setData(utf8Data.c_str(), utf8Data.length());
+
+		if (checkType == true)
+		{
+			doc.setType(MIMEScanner::scanData(utf8Data.c_str(), utf8Data.length()));
+		}
+
 #ifdef DEBUG
 		cout << "FilterUtils::populateDocument: set " << utf8Data.length() << "/" << contentIter->second.length()
 			<< " bytes, converted from charset " << charset << endl;
