@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -23,10 +23,11 @@
 #include <map>
 
 #include "Document.h"
+#include "Visibility.h"
 #include "filters/Filter.h"
 
 /// Utility functions for dealing with Dijon filters.
-class FilterUtils
+class PINOT_EXPORT FilterUtils
 {
 	public:
 		virtual ~FilterUtils();
@@ -47,6 +48,7 @@ class FilterUtils
 		static std::string stripMarkup(const std::string &text);
 
 	protected:
+		static std::set<std::string> m_types;
 		static std::map<std::string, std::string> m_typeAliases;
 
 		FilterUtils();
