@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007,2008 Fabrice Colin
+ *  Copyright 2007-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -53,7 +53,7 @@ using std::map;
 using namespace Dijon;
 
 #ifdef _DYNAMIC_DIJON_FILTERS
-bool get_filter_types(std::set<std::string> &mime_types)
+DIJON_FILTER_EXPORT bool get_filter_types(std::set<std::string> &mime_types)
 {
 #ifdef _DIJON_EXTERNALFILTER_CONFFILE
 	ExternalFilter::initialize(_DIJON_EXTERNALFILTER_CONFFILE, mime_types);
@@ -64,7 +64,7 @@ bool get_filter_types(std::set<std::string> &mime_types)
 	return true;
 }
 
-bool check_filter_data_input(int data_input)
+DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 {
 	Filter::DataInput input = (Filter::DataInput)data_input;
 
@@ -76,7 +76,7 @@ bool check_filter_data_input(int data_input)
 	return false;
 }
 
-Filter *get_filter(const std::string &mime_type)
+DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
 {
 	return new ExternalFilter(mime_type);
 }
