@@ -1,4 +1,6 @@
 /*
+ *  Copyright 2007 Fabrice Colin
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -7,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,6 +23,14 @@
 #include <set>
 #include <map>
 
+#ifndef DIJON_FILTER_EXPORT
+#if defined __GNUC__ && (__GNUC__ >= 4)
+  #define DIJON_FILTER_EXPORT __attribute__ ((visibility("default")))
+#else
+  #define DIJON_FILTER_EXPORT
+#endif
+#endif
+
 namespace Dijon
 {
     class Filter;
@@ -51,7 +61,7 @@ namespace Dijon
         unsigned int, const std::string &);
 
     /// Filter interface.
-    class Filter
+    class DIJON_FILTER_EXPORT Filter
     {
     public:
 	/// Builds an empty filter.
