@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,13 +9,17 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_VSNPRINTF
+#include <stdio.h>
+#include <stdarg.h>
+#endif
 #include <string.h>
 #include <libxml/xmlerror.h>
 #include <libxml/HTMLparser.h>
@@ -42,7 +46,7 @@ using namespace Dijon;
 static const unsigned int HASH_LEN = ((4 * 8 + 5) / 6);
 
 #ifdef _DYNAMIC_DIJON_HTMLFILTER
-bool get_filter_types(std::set<std::string> &mime_types)
+DIJON_FILTER_EXPORT bool get_filter_types(std::set<std::string> &mime_types)
 {
 	mime_types.clear();
 	mime_types.insert("text/html");
@@ -53,7 +57,7 @@ bool get_filter_types(std::set<std::string> &mime_types)
 	return true;
 }
 
-bool check_filter_data_input(int data_input)
+DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 {
 	Filter::DataInput input = (Filter::DataInput)data_input;
 
@@ -66,7 +70,7 @@ bool check_filter_data_input(int data_input)
 	return false;
 }
 
-Filter *get_filter(const std::string &mime_type)
+DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
 {
 	return new HtmlFilter(mime_type);
 }
@@ -509,7 +513,29 @@ static void whitespaceHandler(void *pData, const xmlChar *pText, int txtLen)
 
 static void commentHandler(void *pData, const char *pText)
 {
-	// FIXME: take comments into account, eg on terms position ?
+	if (pData == NULL)
+	{
+		return;
+	}
+
+	HtmlFilter::ParserState *pState = (HtmlFilter::ParserState *)pData;
+	if (pState == NULL)
+	{
+		return;
+	}
+
+	if (pText != NULL)
+	{
+		// Obey htdig noindex
+		if (strncasecmp(pText, "htdig_noindex", 13) == 0)
+		{
+			++pState->m_skip;
+		}
+		else if (strncasecmp(pText, "/htdig_noindex", 14) == 0)
+		{
+			--pState->m_skip;
+		}
+	}
 }
 
 static void errorHandler(void *pData, const char *pMsg, ...)
@@ -525,6 +551,7 @@ static void errorHandler(void *pData, const char *pMsg, ...)
 		return;
 	}
 
+#ifdef HAVE_VSNPRINTF
 	va_list args;
 	char pErr[1000];
 
@@ -534,6 +561,7 @@ static void errorHandler(void *pData, const char *pMsg, ...)
 
 #ifdef DEBUG
 	cout << "HtmlFilter::errorHandler: after " << pState->m_textPos << ": " << pErr << endl;
+#endif
 #endif
 
 	// Be lenient as much as possible
@@ -544,6 +572,7 @@ static void errorHandler(void *pData, const char *pMsg, ...)
 
 static void warningHandler(void *pData, const char *pMsg, ...)
 {
+#ifdef HAVE_VSNPRINTF
 	va_list args;
 	char pErr[1000];
 
@@ -554,6 +583,7 @@ static void warningHandler(void *pData, const char *pMsg, ...)
 #ifdef DEBUG
 	cout << "HtmlFilter::warningHandler: " << pErr << endl;
 #endif
+#endif
 }
 
 Link::Link() :
@@ -907,7 +937,11 @@ bool HtmlFilter::parse_html(const char *pData, unsigned int dataLen)
 	}
 
 	// Append META keywords, if any were found
-	m_pState->m_text += m_pState->m_metaTags["keywords"];
+	map<string, string>::iterator keywordsIter = m_pState->m_metaTags.find("keywords");
+	if (keywordsIter != m_pState->m_metaTags.end())
+	{
+		m_pState->m_text += keywordsIter->second;
+	}
 
 	return true;
 }
