@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -25,7 +25,7 @@ using std::string;
 using namespace Dijon;
 
 #ifdef _DYNAMIC_DIJON_XMLFILTER
-bool get_filter_types(std::set<std::string> &mime_types)
+DIJON_FILTER_EXPORT bool get_filter_types(std::set<std::string> &mime_types)
 {
 	mime_types.clear();
 	mime_types.insert("text/xml");
@@ -34,7 +34,7 @@ bool get_filter_types(std::set<std::string> &mime_types)
 	return true;
 }
 
-bool check_filter_data_input(int data_input)
+DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 {
 	Filter::DataInput input = (Filter::DataInput)data_input;
 
@@ -47,7 +47,7 @@ bool check_filter_data_input(int data_input)
 	return false;
 }
 
-Filter *get_filter(const std::string &mime_type)
+DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
 {
 	return new XmlFilter(mime_type);
 }
