@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -272,6 +272,7 @@ bool DaemonState::crawl_location(const PinotSettings::IndexableLocation &locatio
 	if (start_thread(pScannerThread, true) == true)
 	{
 		++m_crawlers;
+		set_flag(CRAWLING);
 
 		return true;
 	}
@@ -324,6 +325,13 @@ void DaemonState::start(bool forceFullScan)
 	cout << "DaemonState::start: " << m_crawlQueue.size() << " locations to crawl" << endl;
 #endif
 
+	if (m_fullScan == true)
+	{
+		CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
+
+		// Update all items status so that we can get rid of files from deleted sources
+		crawlHistory.updateItemsStatus(CrawlHistory::CRAWLED, CrawlHistory::CRAWLING, 0, true);
+	}
 	// Initiate crawling
 	start_crawling();
 }
@@ -343,12 +351,40 @@ void DaemonState::start_crawling(void)
 			<< m_crawlers << " crawlers" << endl;
 #endif
 		// Get the next location, unless something is still being crawled
-		if ((m_crawlers == 0) &&
-			(m_crawlQueue.empty() == false))
+		if (m_crawlers == 0)
 		{
-			PinotSettings::IndexableLocation nextLocation(m_crawlQueue.front());
+			reset_flag(CRAWLING);
 
-			crawl_location(nextLocation);
+			if (m_crawlQueue.empty() == false)
+			{
+				PinotSettings::IndexableLocation nextLocation(m_crawlQueue.front());
+
+				crawl_location(nextLocation);
+			}
+			else if (m_fullScan == true)
+			{
+				CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
+				set<string> deletedFiles;
+
+				// All files left with status CRAWLING belong to deleted sources
+				if ((m_pDiskHandler != NULL) &&
+					(crawlHistory.getItems(CrawlHistory::CRAWLING, deletedFiles) > 0))
+				{
+#ifdef DEBUG
+					cout << "DaemonState::start_crawling: " << deletedFiles.size() << " orphaned files" << endl;
+#endif
+					for(set<string>::const_iterator fileIter = deletedFiles.begin();
+						fileIter != deletedFiles.end(); ++fileIter)
+					{
+						// Inform the MonitorHandler
+						if (m_pDiskHandler->fileDeleted(fileIter->substr(7)) == true)
+						{
+							// Delete this item
+							crawlHistory.deleteItem(*fileIter);
+						}
+					}
+				}
+			}
 		}
 
 		unlock_lists();
@@ -432,10 +468,10 @@ void DaemonState::on_thread_end(WorkerThread *pThread)
 		if ((errorNum > 0) &&
 			(indexedUrl.empty() == false))
 		{
-			CrawlHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
+			CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 
 			// An entry should already exist for this
-			history.updateItem(indexedUrl, CrawlHistory::ERROR, time(NULL), errorNum);
+			crawlHistory.updateItem(indexedUrl, CrawlHistory::ERROR, time(NULL), errorNum);
 		}
 	}
 	else if (type == "UnindexingThread")
@@ -488,14 +524,13 @@ void DaemonState::on_thread_end(WorkerThread *pThread)
 		settings.clear();
 		settings.loadGlobal(string(SYSCONFDIR) + "/pinot/globalconfig.xml");
 		settings.load();
-
-		// ...and restart everything 
-		start(false);
+		m_reload = false;
 #ifdef DEBUG
-		cout << "DaemonState::on_thread_end: reloaded" << endl;
+		cout << "DaemonState::on_thread_end: reloading" << endl;
 #endif
 
-		m_reload = false;
+		// ...and restart everything 
+		start(true);
 	}
 #ifdef DEBUG
 	cout << "DaemonState::on_thread_end: reload status " << m_reload << endl;
