@@ -9,13 +9,19 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#if _USE_BUTTON_TAB
+#include <gtkmm/rc.h>
+#endif
+
+#include "config.h"
+#include "NLS.h"
 #include "Notebook.h"
 #include "PinotUtils.h"
 
@@ -86,6 +92,8 @@ ResultsTree *ResultsPage::getTree(void) const
 	return m_pTree;
 }
 
+bool NotebookTabBox::m_initialized = false;
+
 NotebookTabBox::NotebookTabBox(const Glib::ustring &title, NotebookPageBox::PageType type) :
 	HBox(),
 	m_title(title),
@@ -93,22 +101,31 @@ NotebookTabBox::NotebookTabBox(const Glib::ustring &title, NotebookPageBox::Page
 	m_tabLabel(NULL),
 	m_tabImage(NULL),
 #if _USE_BUTTON_TAB
-	m_tabImageBox(NULL),
 	m_tabButton(NULL)
 #else
 	m_tabEventBox(NULL)
 #endif
 {
-#if _USE_BUTTON_TAB
-	int width, height;
+	if (m_initialized == false)
+	{
+		m_initialized = true;
 
-	// Lookup the standard icon size
-	bool gotDimensions = IconSize::lookup(ICON_SIZE_MENU, width, height);
+#if _USE_BUTTON_TAB
+		// This was lifted from gnome-terminal's terminal-window.c
+		RC::parse_string("style \"pinot-tab-close-button-style\"\n"
+			"{\n"
+			"GtkWidget::focus-padding = 0\n"
+			"GtkWidget::focus-line-width = 0\n"
+			"xthickness = 0\n"
+			"ythickness = 0\n"
+			"}\n"
+			"widget \"*.pinot-tab-close-button\" style \"pinot-tab-close-button-style\"");
 #endif
+	}
+
 	m_tabLabel = manage(new Label(title));
 	m_tabImage = manage(new Image(StockID("gtk-close"), IconSize(ICON_SIZE_MENU)));
 #if _USE_BUTTON_TAB
-	m_tabImageBox = manage(new HBox());
 	m_tabButton = manage(new Button());
 #else
 	m_tabEventBox = manage(new EventBox);
@@ -125,14 +142,10 @@ NotebookTabBox::NotebookTabBox(const Glib::ustring &title, NotebookPageBox::Page
 #if _USE_BUTTON_TAB
 	m_tabButton->set_relief(RELIEF_NONE);
 	m_tabButton->set_border_width(0);
+	m_tabButton->set_name("pinot-tab-close-button");
+	m_tabButton->set_tooltip_text(_("Close"));
 	m_tabButton->set_alignment(0, 0);
-	// Resize the button
-	if (gotDimensions == true)
-	{
-		m_tabButton->set_size_request(width + 4, height + 4);
-	}
-	m_tabImageBox->pack_start(*m_tabImage, false, true);
-	m_tabButton->add(*m_tabImageBox);
+	m_tabButton->add(*m_tabImage);
 #else
 	m_tabEventBox->add(*m_tabImage);
 	m_tabEventBox->set_events(Gdk::BUTTON_PRESS_MASK);
@@ -148,7 +161,6 @@ NotebookTabBox::NotebookTabBox(const Glib::ustring &title, NotebookPageBox::Page
 	m_tabLabel->show();
 	m_tabImage->show();
 #if _USE_BUTTON_TAB
-	m_tabImageBox->show();
 	m_tabButton->show();
 #else
 	m_tabEventBox->show();
