@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -25,6 +25,7 @@
 #include <gtkmm/paned.h>
 #include <gtkmm/label.h>
 #include <gtkmm/image.h>
+#define _USE_BUTTON_TAB 1
 #if _USE_BUTTON_TAB
 #include <gtkmm/button.h>
 #else
@@ -83,12 +84,12 @@ class NotebookTabBox : public Gtk::HBox
 		sigc::signal2<void, Glib::ustring, NotebookPageBox::PageType>& getCloseSignal(void);
 
 	protected:
+		static bool m_initialized;
 		Glib::ustring m_title;
 		NotebookPageBox::PageType m_pageType;
 		Gtk::Label *m_tabLabel;
 		Gtk::Image *m_tabImage;
 #if _USE_BUTTON_TAB
-		Gtk::HBox *m_tabImageBox;
 		Gtk::Button *m_tabButton;
 #else
 		Gtk::EventBox *m_tabEventBox;
