@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -18,6 +18,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -286,6 +287,7 @@ void PinotSettings::clear(void)
 	m_indexableLocations.clear();
 	m_filePatternsList.clear();
 	m_isBlackList = true;
+	m_editablePluginValues.clear();
 	m_cacheProviders.clear();
 	m_cacheProtocols.clear();
 
@@ -312,6 +314,34 @@ bool PinotSettings::loadGlobal(const string &fileName)
 
 bool PinotSettings::load(void)
 {
+	map<ModuleProperties, bool> engines;
+	string currentUserChannelName(_("Current User"));
+
+	// Some engines are available as back-ends
+	ModuleFactory::getSupportedEngines(engines);
+	for (map<ModuleProperties, bool>::const_iterator engineIter = engines.begin();
+		engineIter != engines.end(); ++engineIter)
+	{
+		if (engineIter->second == true)
+		{
+			string channelName(engineIter->first.m_channel);
+
+			m_engineIds[1 << m_engines.size()] = engineIter->first.m_name;
+
+			// Is a channel specified ?
+			if (channelName.empty() == true)
+			{
+				channelName = currentUserChannelName;
+			}
+			m_engines.insert(engineIter->first);
+
+			if (m_engineChannels.find(channelName) == m_engineChannels.end())
+			{
+				m_engineChannels.insert(pair<string, bool>(channelName, true));
+			}
+		}
+	}
+
 	// Load the configuration file
 	string fileName = getConfigurationFileName();
 	if (m_firstRun == false)
@@ -330,6 +360,7 @@ bool PinotSettings::load(void)
 		m_labels.insert(_("New"));
 		m_labels.insert(_("Personal"));
 		m_isBlackList = getDefaultPatterns(m_filePatternsList);
+
 		// Create default queries
 		struct passwd *pPasswd = getpwuid(geteuid());
 		if (pPasswd != NULL)
@@ -354,25 +385,15 @@ bool PinotSettings::load(void)
 				addQuery(queryProps);
 			}
 		}
-	}
 
-	// Some search engines are hardcoded
-#ifdef HAVE_GOOGLEAPI
-	m_engineIds[1 << m_engines.size()] = "Google API";
-	m_engines.insert(Engine("Google API", "googleapi", "", "The Web"));
-	m_engineChannels.insert(pair<string, bool>("The Web", true));
-#endif
-	// Others are available as back-ends
-	map<string, bool> engines;
-	ModuleFactory::getSupportedEngines(engines);
-	for (map<string, bool>::const_iterator engineIter = engines.begin();
-		engineIter != engines.end(); ++engineIter)
-	{
-		if (engineIter->second == true)
-		{
-			m_engineIds[1 << m_engines.size()] = engineIter->first;
-			m_engines.insert(Engine(engineIter->first, engineIter->first, "", ""));
-		}
+		QueryProperties queryProps(_("Latest First"), "dir:/");
+		queryProps.setSortOrder(QueryProperties::DATE);
+		addQuery(queryProps);
+		addQuery(QueryProperties(_("Home Stuff"), string("dir:") + getHomeDirectory()));
+		addQuery(QueryProperties(_("With Label New"), "label:New"));
+		addQuery(QueryProperties(_("Have CJKV"), "tokens:CJKV"));
+		addQuery(QueryProperties(_("In English"), "lang:en"));
+		addQuery(QueryProperties(_("10kb And Smaller"), "0..10240b"));
 	}
 
 	return true;
@@ -529,6 +550,10 @@ bool PinotSettings::loadConfiguration(const std::string &fileName, bool isGlobal
 				{
 					loadFilePatterns(pElem);
 				}
+				else if (nodeName == "pluginparameters")
+				{
+					loadPluginParameters(pElem);
+				}
 			}
 		}
 	}
@@ -1161,6 +1186,53 @@ bool PinotSettings::loadFilePatterns(const Element *pElem)
 	return true;
 }
 
+bool PinotSettings::loadPluginParameters(const Element *pElem)
+{
+	if (pElem == NULL)
+	{
+		return false;
+	}
+
+	Node::NodeList childNodes = pElem->get_children();
+	if (childNodes.empty() == true)
+	{
+		return false;
+	}
+
+	string name, value;
+
+	// Load the plugin parameters' values 
+	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
+	{
+		Node *pNode = (*iter);
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
+		{
+			continue;
+		}
+
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
+
+		if (nodeName == "name")
+		{
+			name = nodeContent;
+		}
+		else if (nodeName == "value")
+		{
+			value = nodeContent;
+		}
+	}
+
+	map<string, string>::iterator valueIter = m_editablePluginValues.find(name);
+	if (valueIter != m_editablePluginValues.end())
+	{
+		valueIter->second = value;
+	}
+
+	return true;
+}
+
 bool PinotSettings::loadCacheProviders(const Element *pElem)
 {
 	if (pElem == NULL)
@@ -1259,18 +1331,29 @@ bool PinotSettings::loadSearchEngines(const string &directoryName)
 			if ((stat(location.c_str(), &fileStat) == 0) &&
 				(S_ISREG(fileStat.st_mode)))
 			{
-				string engineName, engineChannel;
+				SearchPluginProperties properties;
 
-				if ((PluginWebEngine::getDetails(location, engineName, engineChannel) == true) &&
-					(engineName.empty() == false))
+				if ((PluginWebEngine::getDetails(location, properties) == true) &&
+					(properties.m_name.empty() == false))
 				{
-					m_engineIds[1 << m_engines.size()] = engineName;
-					if (engineChannel.empty() == true)
+					m_engineIds[1 << m_engines.size()] = properties.m_name;
+					if (properties.m_channel.empty() == true)
 					{
-						engineChannel = _("Unclassified");
+						properties.m_channel = _("Unclassified");
 					}
-					m_engines.insert(Engine(engineName, "sherlock", location, engineChannel));
-					m_engineChannels.insert(pair<string, bool>(engineChannel, true));
+					m_engines.insert(ModuleProperties("sherlock", properties.m_name, location, properties.m_channel));
+					m_engineChannels.insert(pair<string, bool>(properties.m_channel, true));
+
+					// Any editable parameters in this plugin ?
+					for (map<string, string>::const_iterator editableIter = properties.m_editableParameters.begin();
+						editableIter != properties.m_editableParameters.end(); ++editableIter)
+					{
+						m_editablePluginValues[editableIter->second] = "";
+					}
+#ifdef DEBUG
+					cout << "PinotSettings::loadSearchEngines: " << properties.m_name
+						<< " has " << properties.m_editableParameters.size() << " editable values" << endl;
+#endif
 				}
 			}
 		}
@@ -1448,6 +1531,23 @@ bool PinotSettings::save(void)
 			addChildElement(pElem, "pattern", *patternIter);
 		}
 		addChildElement(pElem, "forbid", (m_isBlackList ? "YES" : "NO"));
+		// Values of editable plugin parameters
+		for (map<string, string>::iterator editableIter = m_editablePluginValues.begin();
+			editableIter != m_editablePluginValues.end() ; ++editableIter)
+		{
+			if (editableIter->second.empty() == true)
+			{
+				continue;
+			}
+
+			pElem = pRootElem->add_child("pluginparameters");
+			if (pElem == NULL)
+			{
+				return false;
+			}
+			addChildElement(pElem, "name", editableIter->first);
+			addChildElement(pElem, "value", editableIter->second);
+		}
 
 		// Save to file
 		doc.write_to_file_formatted(getConfigurationFileName());
@@ -1609,7 +1709,7 @@ IndexInterface *PinotSettings::getIndex(const string &location)
 }
 
 /// Returns the search engines set.
-bool PinotSettings::getSearchEngines(set<PinotSettings::Engine> &engines, string channelName) const
+bool PinotSettings::getSearchEngines(set<ModuleProperties> &engines, const string &channelName) const
 {
 	if (channelName.empty() == true)
 	{
@@ -1624,8 +1724,8 @@ bool PinotSettings::getSearchEngines(set<PinotSettings::Engine> &engines, string
 			return false;
 		}
 
-		// Copy channels that belong to the given channel
-		for (set<Engine>::iterator engineIter = m_engines.begin(); engineIter != m_engines.end(); ++engineIter)
+		// Copy engines that belong to the given channel
+		for (set<ModuleProperties>::iterator engineIter = m_engines.begin(); engineIter != m_engines.end(); ++engineIter)
 		{
 			if (engineIter->m_channel == channelName)
 			{
@@ -1743,6 +1843,7 @@ bool PinotSettings::getDefaultPatterns(set<ustring> &defaultPatterns)
 	defaultPatterns.insert("*.a");
 	defaultPatterns.insert("*.asf");
 	defaultPatterns.insert("*.avi");
+	defaultPatterns.insert("*CVS");
 	defaultPatterns.insert("*.cap");
 	defaultPatterns.insert("*.divx");
 	defaultPatterns.insert("*.flv");
@@ -1760,7 +1861,9 @@ bool PinotSettings::getDefaultPatterns(set<ustring> &defaultPatterns)
 	defaultPatterns.insert("*.rar");
 	defaultPatterns.insert("*.sh");
 	defaultPatterns.insert("*.so");
+	defaultPatterns.insert("*.svn");
 	defaultPatterns.insert("*.tiff");
+	defaultPatterns.insert("*.torrent");
 	defaultPatterns.insert("*.wmv");
 	defaultPatterns.insert("*.xbm");
 	defaultPatterns.insert("*.xpm");
@@ -1799,63 +1902,6 @@ bool PinotSettings::isBlackListed(const string &fileName)
 	return !m_isBlackList;
 }
 
-PinotSettings::Engine::Engine()
-{
-}
-
-PinotSettings::Engine::Engine(string name, string type, string option, string channel) :
-	m_name(name),
-	m_type(type),
-	m_option(option),
-	m_channel(channel)
-{
-}
-
-PinotSettings::Engine::Engine(const Engine &other) :
-	m_name(other.m_name),
-	m_type(other.m_type),
-	m_option(other.m_option),
-	m_channel(other.m_channel)
-{
-}
-
-PinotSettings::Engine::~Engine()
-{
-}
-
-PinotSettings::Engine &PinotSettings::Engine::operator=(const PinotSettings::Engine &other)
-{
-	if (this != &other)
-	{
-		m_name = other.m_name;
-		m_type = other.m_type;
-		m_option = other.m_option;
-		m_channel = other.m_channel;
-	}
-
-	return *this;
-}
-
-bool PinotSettings::Engine::operator<(const PinotSettings::Engine &other) const
-{
-	if (m_name < other.m_name)
-	{
-		return true;
-	}
-
-	return false;
-}
-
-bool PinotSettings::Engine::operator==(const Engine &other) const
-{
-	if (m_name == other.m_name)
-	{
-		return true;
-	}
-
-	return false;
-}
-
 PinotSettings::TimestampedItem::TimestampedItem() :
 	m_modTime(0)
 {
