@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -31,8 +31,8 @@
 #include "TimeConverter.h"
 #include "Url.h"
 #include "QueryHistory.h"
-#include "ResultsExporter.h"
 #include "ViewHistory.h"
+#include "ResultsExporter.h"
 #include "config.h"
 #include "NLS.h"
 #include "PinotSettings.h"
@@ -52,7 +52,7 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	m_pResultsScrolledwindow(NULL),
 	m_settings(settings),
 	m_pExtractScrolledwindow(NULL),
-	m_extractTreeView(NULL),
+	m_extractTextView(NULL),
 	m_showExtract(true),
 	m_groupMode(groupMode)
 {
@@ -60,7 +60,7 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 
 	m_pResultsScrolledwindow = manage(new ScrolledWindow());
 	m_pExtractScrolledwindow = manage(new ScrolledWindow());
-	m_extractTreeView = manage(new TreeView());
+	m_extractTextView = manage(new TextView());
 
 	// This is the actual results tree
 	set_events(Gdk::BUTTON_PRESS_MASK);
@@ -79,20 +79,30 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	m_pResultsScrolledwindow->add(*this);
 
 	// That's for the extract view
-	m_extractTreeView->set_events(Gdk::BUTTON_PRESS_MASK);
-	m_extractTreeView->set_flags(CAN_FOCUS);
-	m_extractTreeView->set_headers_clickable(false);
-	m_extractTreeView->set_headers_visible(false);
-	m_extractTreeView->set_rules_hint(true);
-	m_extractTreeView->set_reorderable(false);
-	m_extractTreeView->set_enable_search(true);
-	m_extractTreeView->get_selection()->set_mode(SELECTION_SINGLE);
+	m_extractTextView->set_flags(CAN_FOCUS);
+	m_extractTextView->set_editable(false);
+	m_extractTextView->set_cursor_visible(false);
+	m_extractTextView->set_pixels_above_lines(0);
+	m_extractTextView->set_pixels_below_lines(0);
+	m_extractTextView->set_pixels_inside_wrap(0);
+	m_extractTextView->set_left_margin(0);
+	m_extractTextView->set_right_margin(0);
+	m_extractTextView->set_indent(0);
+	m_extractTextView->set_wrap_mode(WRAP_WORD);
+	m_extractTextView->set_justification(JUSTIFY_LEFT);
+	RefPtr<TextTag> refBoldTag = TextBuffer::Tag::create("bold");
+	refBoldTag->property_weight() = 900;
+	RefPtr<TextBuffer> refBuffer = m_extractTextView->get_buffer();
+	if (refBuffer)
+	{
+		refBuffer->get_tag_table()->add(refBoldTag); 
+	}
 	m_pExtractScrolledwindow->set_flags(CAN_FOCUS);
 	m_pExtractScrolledwindow->set_border_width(4);
 	m_pExtractScrolledwindow->set_shadow_type(SHADOW_NONE);
 	m_pExtractScrolledwindow->set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
 	m_pExtractScrolledwindow->property_window_placement().set_value(CORNER_TOP_LEFT);
-	m_pExtractScrolledwindow->add(*m_extractTreeView);
+	m_pExtractScrolledwindow->add(*m_extractTextView);
 
 	// Associate the columns model to the results tree
 	m_refStore = TreeStore::create(m_resultsColumns);
@@ -173,21 +183,10 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	m_upIconPixbuf = render_icon(Stock::GO_UP, ICON_SIZE_MENU, "MetaSE-pinot");
 	m_downIconPixbuf = render_icon(Stock::GO_DOWN, ICON_SIZE_MENU, "MetaSE-pinot");
 
-	// Associate the columns model to the extract tree
-	m_refExtractStore = ListStore::create(m_extractColumns);
-	m_extractTreeView->set_model(m_refExtractStore);
-	pColumn = new TreeViewColumn(_("Extract"));
-	pTextRenderer = new CellRendererText();
-	pColumn->pack_start(*manage(pTextRenderer));
-	pColumn->set_cell_data_func(*pTextRenderer, sigc::mem_fun(*this, &ResultsTree::renderExtractColumn));
-	pColumn->add_attribute(pTextRenderer->property_text(), m_extractColumns.m_name);
-	pColumn->set_sizing(TREE_VIEW_COLUMN_AUTOSIZE);
-	m_extractTreeView->append_column(*manage(pColumn));
-
 	// Show all
 	show();
 	m_pResultsScrolledwindow->show();
-	m_extractTreeView->show();
+	m_extractTextView->show();
 	m_pExtractScrolledwindow->show();
 }
 
@@ -377,8 +376,11 @@ bool ResultsTree::onSelectionSelect(const RefPtr<TreeModel>& model,
 		(m_groupMode != FLAT))
 	{
 		// Clear the extract
-		m_extractTreeView->get_selection()->unselect_all();
-		m_refExtractStore->clear();
+		RefPtr<TextBuffer> refBuffer = m_extractTextView->get_buffer();
+		if (refBuffer)
+		{
+			refBuffer->set_text("");
+		}
 
 		// Is this an actual result ?
 		ResultsModelColumns::RowType type = row[m_resultsColumns.m_resultType];
@@ -387,10 +389,42 @@ bool ResultsTree::onSelectionSelect(const RefPtr<TreeModel>& model,
 #ifdef DEBUG
 			cout << "ResultsTree::onSelectionSelect: extract for " << row[m_resultsColumns.m_text] << endl;
 #endif
+			RefPtr<TextBuffer> refBuffer = m_extractTextView->get_buffer();
+			if (refBuffer)
+			{
+				ustring extract(findResultsExtract(row));
+				TextBuffer::iterator bufferPos = refBuffer->begin();
+				ustring::size_type textPos = 0, boldPos = extract.find("<b>");
 
-			TreeModel::iterator extractIter = m_refExtractStore->append();
-			TreeModel::Row extractRow = *extractIter;
-			extractRow[m_extractColumns.m_name] = findResultsExtract(row);
+				if (boldPos == ustring::npos)
+				{
+					refBuffer->set_text(extract);
+				}
+				else
+				{
+					while (boldPos != ustring::npos)
+					{
+						bufferPos = refBuffer->insert(bufferPos, extract.substr(textPos, boldPos - textPos));
+
+						textPos = boldPos + 3;
+						boldPos = extract.find("</b>", textPos);
+						if (boldPos == ustring::npos)
+						{
+							continue;
+						}
+						bufferPos = refBuffer->insert_with_tag(bufferPos, extract.substr(textPos, boldPos - textPos), "bold");
+
+						// Next
+						textPos = boldPos + 4;
+						boldPos = extract.find("<b>", textPos);
+					}
+
+					if (textPos + 1 < extract.length())
+					{
+						bufferPos = refBuffer->insert(bufferPos, extract.substr(textPos, boldPos - textPos));
+					}
+				}
+			}
 		}
 	}
 
@@ -428,9 +462,9 @@ ScrolledWindow *ResultsTree::getExtractScrolledWindow(void) const
 //
 // Returns the extract tree.
 //
-TreeView *ResultsTree::getExtractTree(void) const
+bool ResultsTree::focusOnExtract(void) const
 {
-	return m_extractTreeView;
+	return m_extractTextView->is_focus();
 }
 
 //
@@ -440,13 +474,10 @@ ustring ResultsTree::getExtract(void) const
 {
 	ustring text;
 
-	TreeModel::Children children = m_refExtractStore->children();
-	for (TreeModel::Children::iterator iter = children.begin();
-		iter != children.end(); ++iter)
+	RefPtr<TextBuffer> refBuffer = m_extractTextView->get_buffer();
+	if (refBuffer)
 	{
-		TreeModel::Row row = *iter;
-
-		text += row[m_extractColumns.m_name];
+		text = refBuffer->get_text();
 	}
 
 	return text;
@@ -475,50 +506,66 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 		rootType = ResultsModelColumns::ROW_HOST;
 	}
 
-	// Find out what the search engine ID is
 	unsigned int indexId = 0;
-	unsigned int engineId = m_settings.getEngineId(engineName);
-	if (engineId == 0)
+	unsigned int engineId = 0;
+
+	// Find out what the search engine ID is
+	if (engineName.empty() == false)
 	{
-		// Chances are this engine is an index
-		std::map<string, string>::const_iterator mapIter = m_settings.getIndexes().find(engineName);
-		if (mapIter != m_settings.getIndexes().end())
+		engineId = m_settings.getEngineId(engineName);
+		if (engineId == 0)
 		{
-			// Yes, it is
-			indexId = m_settings.getIndexIdByName(engineName);
-			engineId = m_settings.getEngineId(m_settings.m_defaultBackend);
+			// Chances are this engine is an index
+			std::map<string, string>::const_iterator mapIter = m_settings.getIndexes().find(engineName);
+			if (mapIter != m_settings.getIndexes().end())
+			{
+				// Yes, it is
+				indexId = m_settings.getIndexIdByName(engineName);
+				engineId = m_settings.getEngineId(m_settings.m_defaultBackend);
 #ifdef DEBUG
-			cout << "ResultsTree::addResults: engine is index " << engineName << " " << indexId << " " << engineId << endl;
+				cout << "ResultsTree::addResults: engine is index " << engineName << " " << indexId << " " << engineId << endl;
+#endif
+			}
+#ifdef DEBUG
+			else cout << "ResultsTree::addResults: " << engineName << " is not an index" <<  endl;
 #endif
 		}
 #ifdef DEBUG
-		else cout << "ResultsTree::addResults: " << engineName << " is not an index" <<  endl;
+		else cout << "ResultsTree::addResults: ID for engine " << engineName << " is " << engineId <<  endl;
 #endif
 	}
-#ifdef DEBUG
-	else cout << "ResultsTree::addResults: ID for engine " << engineName << " is " << engineId <<  endl;
-#endif
 
 	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 	ViewHistory viewHistory(m_settings.getHistoryDatabaseName());
-	string lastRun(queryHistory.getLastRun(m_treeName, engineName));
-	time_t lastRunTime = 0;
+	set<time_t> latestRuns;
+	time_t secondLastRunTime = 0;
 	bool isNewQuery = false;
 
 	// Is this a new query ?
-	if (lastRun.empty() == true)
+	if ((queryHistory.getLatestRuns(m_treeName, engineName, 2, latestRuns) == false) ||
+		(latestRuns.empty() == true))
 	{
 		isNewQuery = true;
 	}
 	else
 	{
-		lastRunTime = TimeConverter::fromTimestamp(lastRun);
+		set<time_t>::const_iterator runIter = latestRuns.begin();
+
+		// We only need to keep the last two runs
+		if (runIter != latestRuns.end())
+		{
+			++runIter;
+			if (runIter != latestRuns.end())
+			{
+				secondLastRunTime = (*runIter);
+			}
+		}
 	}
+#ifdef DEBUG
+	cout << "ResultsTree::addResults: " << resultsList.size() << " results, second last run " << secondLastRunTime << endl;
+#endif
 
 	// Look at the results list
-#ifdef DEBUG
-	cout << "ResultsTree::addResults: " << resultsList.size() << " results, last run " << lastRun << endl;
-#endif
 	for (vector<DocumentInfo>::const_iterator resultIter = resultsList.begin();
 		resultIter != resultsList.end(); ++resultIter)
 	{
@@ -549,24 +596,17 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 			float oldestScore = 0;
 			float previousScore = queryHistory.hasItem(m_treeName, engineName,
 				location, oldestScore);
+#ifdef DEBUG
+			cout << "ResultsTree::addResults: " << location << " has scores "
+				<< previousScore << ", " << oldestScore << endl;
+#endif
 			if (previousScore > 0)
 			{
-				if (updateHistory == true)
-				{
-					// Update this result whatever the current and previous rankings were
-					queryHistory.updateItem(m_treeName, engineName, location,
-						title, extract, charset, currentScore);
-				}
+				// Yes, it has
 				rankDiff = (int)(currentScore - previousScore);
 			}
 			else
 			{
-				// No, this is a new result
-				if (updateHistory == true)
-				{
-					queryHistory.insertItem(m_treeName, engineName, location,
-						resultIter->getTitle(), extract, charset, currentScore);
-				}
 				// New results are displayed as such only if the query has already been run on the engine
 				if (isNewQuery == false)
 				{
@@ -574,6 +614,12 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 					rankDiff = 666;
 				}
 			}
+
+			if (updateHistory == true)
+			{
+				queryHistory.insertItem(m_treeName, engineName, location,
+					title, extract, currentScore);
+			}
 		}
 
 		++count;
@@ -617,9 +663,9 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 	{
 #ifdef DEBUG
 		cout << "ResultsTree::addResults: removing items for " << m_treeName
-			<< ", " << engineName << " older than " << lastRunTime << endl;
+			<< ", " << engineName << " older than " << secondLastRunTime << endl;
 #endif
-		queryHistory.deleteItems(m_treeName, engineName, lastRunTime);
+		queryHistory.deleteItems(m_treeName, engineName, secondLastRunTime);
 	}
 
 	if (count > 0)
@@ -1193,8 +1239,11 @@ void ResultsTree::clear(void)
 		m_refStore->clear();
 
 		// Clear the extract
-		m_extractTreeView->get_selection()->unselect_all();
-		m_refExtractStore->clear();
+		RefPtr<TextBuffer> refBuffer = m_extractTextView->get_buffer();
+		if (refBuffer)
+		{
+			refBuffer->set_text("");
+		}
 
 		onSelectionChanged();
 	}
@@ -1284,7 +1333,7 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 			set<string> engineNames, indexNames;
 			TreeModel::Row childRow = *childIter;
 			DocumentInfo result;
-			string engineName, charset, serial(from_utf8(childRow[m_resultsColumns.m_serial]));
+			string engineName, serial(from_utf8(childRow[m_resultsColumns.m_serial]));
 			unsigned int engineIds = childRow[m_resultsColumns.m_engines];
 			unsigned int indexIds = childRow[m_resultsColumns.m_indexes];
 
@@ -1310,7 +1359,7 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 			if (m_groupMode != FLAT)
 			{
 				result.setExtract(queryHistory.getItemExtract(from_utf8(m_treeName),
-					engineName, result.getLocation(), charset));
+					engineName, result.getLocation()));
 			}
 			result.setTimestamp(from_utf8(childRow[m_resultsColumns.m_timestamp]));
 
@@ -1528,7 +1577,7 @@ ustring ResultsTree::findResultsExtract(const Gtk::TreeModel::Row &row)
 	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 	set<string> engineNames, indexNames;
 	string url(from_utf8(row[m_resultsColumns.m_url]));
-	string extract, charset;
+	string extract;
 	unsigned int engineIds = row[m_resultsColumns.m_engines];
 	unsigned int indexIds = row[m_resultsColumns.m_indexes];
 
@@ -1561,11 +1610,11 @@ ustring ResultsTree::findResultsExtract(const Gtk::TreeModel::Row &row)
 #ifdef DEBUG
 			cout << "ResultsTree::findResultsExtract: engine or index " << engineName << endl;
 #endif
-			extract = queryHistory.getItemExtract(from_utf8(m_treeName), engineName, url, charset);
+			extract = queryHistory.getItemExtract(from_utf8(m_treeName), engineName, url);
 			if (extract.empty() == false)
 			{
 				// Stop here
-				return to_utf8(extract, charset);
+				return extract;
 			}
 		}
 	}
