@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -69,10 +69,10 @@ class DirectoryScannerThread : public WorkerThread
 		sigc::signal3<void, DocumentInfo, std::string, bool> m_signalFileFound;
 		std::map<std::string, time_t> m_updateCache;
 
-		void cacheUpdate(const std::string &location, time_t mTime, CrawlHistory &history);
-		void flushUpdates(CrawlHistory &history);
+		void cacheUpdate(const std::string &location, time_t mTime, CrawlHistory &crawlHistory);
+		void flushUpdates(CrawlHistory &crawlHistory);
 		void foundFile(const DocumentInfo &docInfo);
-		bool scanEntry(const std::string &entryName, CrawlHistory &history);
+		bool scanEntry(const std::string &entryName, CrawlHistory &crawlHistory);
 		virtual void doWork(void);
 
 	private:
