@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -38,6 +38,7 @@
 #include "MIMEScanner.h"
 #include "StringManip.h"
 #include "TimeConverter.h"
+#include "Timer.h"
 #include "Url.h"
 #include "HtmlFilter.h"
 #include "FilterUtils.h"
@@ -268,11 +269,12 @@ void WorkerThread::emitSignal(void)
 }
 
 ThreadsManager::ThreadsManager(const string &defaultIndexLocation,
-	unsigned int maxIndexThreads) :
+	unsigned int maxIndexThreads, unsigned int maxThreadsTime) :
 	m_defaultIndexLocation(defaultIndexLocation),
 	m_maxIndexThreads(maxIndexThreads),
 	m_nextThreadId(1),
 	m_backgroundThreadsCount(0),
+	m_foregroundThreadsMaxTime(maxThreadsTime),
 	m_numCPUs(1),
 	m_stopIndexing(false)
 {
@@ -362,9 +364,9 @@ WorkerThread *ThreadsManager::get_thread(void)
 
 				// Foreground threads ought not to run very long
 				if ((threadIter->second->isBackground() == false) &&
-					(threadIter->second->getStartTime() + 300 < timeNow))
+					(threadIter->second->getStartTime() + m_foregroundThreadsMaxTime < timeNow))
 				{
-					// This thread has been running for more than 5 minutes already !
+					// This thread has been running for too long !
 					threadIter->second->stop();
 
 					cerr << "Stopped long-running thread " << threadId << endl;
@@ -1092,6 +1094,8 @@ void EngineQueryThread::doWork(void)
 			pDownloader->setSetting("proxyport", portStr);
 			pDownloader->setSetting("proxytype", settings.m_proxyType);
 		}
+
+		pWebEngine->setEditableValues(settings.m_editablePluginValues);
 	}
 
 	if (m_listingIndex == false)
@@ -1154,6 +1158,8 @@ EngineHistoryThread::EngineHistoryThread(const string &engineDisplayableName,
 #ifdef DEBUG
 	cout << "EngineHistoryThread::EngineHistoryThread: engine " << m_engineDisplayableName << endl;
 #endif
+	// Results are converted to UTF-8 prior to insertion in the history database
+	m_resultsCharset = "UTF-8";
 	m_isLive = false;
 }
 
@@ -1163,9 +1169,9 @@ EngineHistoryThread::~EngineHistoryThread()
 
 void EngineHistoryThread::doWork(void)
 {
-	QueryHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
+	QueryHistory queryHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 
-	if (history.getItems(m_queryProps.getName(), m_engineDisplayableName,
+	if (queryHistory.getItems(m_queryProps.getName(), m_engineDisplayableName,
 		m_maxDocsCount, m_documentsList) == false)
 	{
 		m_errorNum = HISTORY_FAILED;
@@ -1174,8 +1180,8 @@ void EngineHistoryThread::doWork(void)
 	else if (m_documentsList.empty() == false)
 	{
 		// Get the first result's charset
-		history.getItemExtract(m_queryProps.getName(), m_engineDisplayableName,
-			m_documentsList.front().getLocation(), m_resultsCharset);
+		queryHistory.getItemExtract(m_queryProps.getName(), m_engineDisplayableName,
+			m_documentsList.front().getLocation());
 	}
 }
 
@@ -1274,6 +1280,16 @@ string LabelUpdateThread::getType(void) const
 	return "LabelUpdateThread";
 }
 
+bool LabelUpdateThread::modifiedDocsIndex(void) const
+{
+	return !m_docsIds.empty();
+}
+
+bool LabelUpdateThread::modifiedDaemonIndex(void) const
+{
+	return !m_daemonIds.empty();
+}
+
 void LabelUpdateThread::doWork(void)
 {
 	bool actOnDocuments = false;
@@ -1387,6 +1403,7 @@ void DownloadingThread::doWork(void)
 	}
 	else if (m_done == false)
 	{
+		Timer collectTimer;
 		PinotSettings &settings = PinotSettings::getInstance();
 
 		// Set up the proxy
@@ -1401,7 +1418,11 @@ void DownloadingThread::doWork(void)
 			m_pDownloader->setSetting("proxytype", settings.m_proxyType);
 		}
 
+		collectTimer.start();
+
 		m_pDoc = m_pDownloader->retrieveUrl(m_docInfo);
+
+		cout << "Retrieved " << m_docInfo.getLocation() << " in " << collectTimer.stop() << " ms" << endl;
 	}
 
 	if (m_pDoc == NULL)
@@ -1945,7 +1966,7 @@ void MonitorThread::stop(void)
 
 void MonitorThread::processEvents(void)
 {
-	CrawlHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
+	CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 	queue<MonitorEvent> events;
 
 #ifdef DEBUG
@@ -2021,7 +2042,7 @@ void MonitorThread::processEvents(void)
 				{
 					m_pHandler->fileModified(event.m_location);
 				}
-				else if (history.hasItem("file://" + event.m_location, status, itemDate) == true)
+				else if (crawlHistory.hasItem("file://" + event.m_location, status, itemDate) == true)
 				{
 					// Was the file actually modified ?
 					if ((stat(event.m_location.c_str(), &fileStat) == 0) &&
