@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -109,7 +109,8 @@ class ThreadsManager : virtual public sigc::trackable
 {
 	public:
 		ThreadsManager(const std::string &defaultIndexLocation,
-			unsigned int maxIndexThreads);
+			unsigned int maxIndexThreads,
+			unsigned int maxThreadsTime = 300);
 		virtual ~ThreadsManager();
 
 		bool start_thread(WorkerThread *pWorkerThread, bool inBackground = false);
@@ -143,6 +144,7 @@ class ThreadsManager : virtual public sigc::trackable
 		unsigned int m_maxIndexThreads;
 		unsigned int m_nextThreadId;
 		unsigned int m_backgroundThreadsCount;
+		unsigned int m_foregroundThreadsMaxTime;
 		long m_numCPUs;
 		sigc::signal1<void, WorkerThread *> m_onThreadEndSignal;
 		std::set<std::string> m_beingIndexed;
@@ -334,6 +336,10 @@ class LabelUpdateThread : public WorkerThread
 
 		virtual std::string getType(void) const;
 
+		bool modifiedDocsIndex(void) const;
+
+		bool modifiedDaemonIndex(void) const;
+
 	protected:
 		std::set<std::string> m_labelsToAdd;
 		std::set<std::string> m_labelsToDelete;
