@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2008 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -34,12 +34,12 @@
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/clipboard.h>
 #include <gtkmm/recentmanager.h>
+#include <gtkmm/targetlist.h>
 #include <gtkmm/main.h>
 
 #include "config.h"
 #include "NLS.h"
 #include "CommandLine.h"
-#include "StringManip.h"
 #include "TimeConverter.h"
 #include "MIMEScanner.h"
 #include "Url.h"
@@ -92,7 +92,7 @@ unsigned int mainWindow::m_maxDocsCount = 100;
 unsigned int mainWindow::m_maxIndexThreads = 2;
 
 mainWindow::InternalState::InternalState(unsigned int maxIndexThreads, mainWindow *pWindow) :
-	ThreadsManager(PinotSettings::getInstance().m_docsIndexLocation, maxIndexThreads),
+	ThreadsManager(PinotSettings::getInstance().m_docsIndexLocation, maxIndexThreads, 60),
 	m_liveQueryLength(0),
 	m_currentPage(0),
 	m_browsingIndex(false)
@@ -179,6 +179,19 @@ mainWindow::mainWindow() :
 	// Connect to the "changed" signal
 	queryTreeview->get_selection()->signal_changed().connect(
 		sigc::mem_fun(*this, &mainWindow::on_queryTreeviewSelection_changed));
+
+	// Specify drop targets
+	queryTreeview->drag_dest_set(Gtk::DEST_DEFAULT_ALL, DragAction(GDK_ACTION_COPY|GDK_ACTION_MOVE));
+	// FIXME: this causes some text files to be received as text, others as URIs !
+	//queryTreeview->drag_dest_add_text_targets();
+	queryTreeview->drag_dest_add_uri_targets();
+	RefPtr<TargetList> targetList = queryTreeview->drag_dest_get_target_list();
+	targetList->add("UTF8_STRING", TargetFlags(0), 0);
+	targetList->add("text/plain", TargetFlags(0), 0);
+	// Connect to the drag data received signal
+	queryTreeview->signal_drag_data_received().connect(
+		sigc::mem_fun(*this, &mainWindow::on_data_received));
+
 	// Populate
 	populate_queryTreeview("");
 
@@ -291,7 +304,7 @@ mainWindow::~mainWindow()
 //
 void mainWindow::populate_queryTreeview(const string &selectedQueryName)
 {
-	QueryHistory history(m_settings.getHistoryDatabaseName());
+	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 	const std::map<string, QueryProperties> &queries = m_settings.getQueries();
 
 	// Reset the whole tree
@@ -304,22 +317,29 @@ void mainWindow::populate_queryTreeview(const string &selectedQueryName)
 	{
 		TreeModel::iterator iter = m_refQueryTree->append();
 		TreeModel::Row row = *iter;
-		string queryName = queryIter->first;
-		string lastRun = history.getLastRun(queryName);
+		set<time_t> latestRuns;
+		string queryName(queryIter->first);
+		string lastRun;
 		time_t lastRunTime = 0;
 
 		row[m_queryColumns.m_name] = to_utf8(queryName);
-		if (lastRun.empty() == true)
+		if ((queryHistory.getLatestRuns(queryName, "", 1, latestRuns) == false) ||
+			(latestRuns.empty() == true))
 		{
 			lastRun = _("N/A");
 		}
 		else
 		{
-			lastRunTime = TimeConverter::fromTimestamp(lastRun);
+			lastRunTime = *(latestRuns.begin());
+			lastRun = TimeConverter::toTimestamp(lastRunTime);
+#ifdef DEBUG
+			cout << "mainWindow::populate_queryTreeview: latest run of "
+				<< queryName << " is " << lastRun << "/" << lastRunTime << endl;
+#endif
 		}
 		row[m_queryColumns.m_lastRun] = to_utf8(lastRun);
 		row[m_queryColumns.m_lastRunTime] = lastRunTime;
-		ustring summary(StringManip::replaceSubString(queryIter->second.getFreeQuery(), "\n", " "));
+		ustring summary(queryIter->second.getFreeQuery());
 		if (summary.empty() == false)
 		{
 			row[m_queryColumns.m_summary] = summary;
@@ -494,9 +514,15 @@ void mainWindow::add_query(QueryProperties &queryProps, bool mergeQueries)
 				QueryProperties existingQueryProps = row[m_queryColumns.m_properties];
 				ustring freeQuery(existingQueryProps.getFreeQuery());
 
-				if (freeQuery != queryProps.getFreeQuery())
+				// Only merge if the new bit isn't in already
+				if (freeQuery.find(queryProps.getFreeQuery()) == string::npos)
 				{
-					queryProps.setFreeQuery("( " + freeQuery + " ) or ( " + queryProps.getFreeQuery() + " )");
+					queryProps.setFreeQuery("( " + freeQuery + " ) OR ( " + queryProps.getFreeQuery() + " )");
+				}
+				else
+				{
+					// Leave the query as it is
+					return;
 				}
 			}
 			m_settings.removeQuery(queryName);
@@ -592,23 +618,83 @@ bool mainWindow::get_results_page_details(const ustring &queryName,
 }
 
 //
-// Query list selection changed
+// Drag and drop
 //
-void mainWindow::on_queryTreeviewSelection_changed()
+void mainWindow::on_data_received(const RefPtr<DragContext> &context,
+	int x, int y, const SelectionData &data, guint info, guint time)
 {
-	bool enableButtons = false;
+	list<ustring> droppedUris = data.get_uris();
+	ustring droppedText(data.get_text());
+	bool goodDrop = false;
 
-	TreeModel::iterator iter = queryTreeview->get_selection()->get_selected();
-	// Anything selected ?
-	if (iter)
+	cout << "mainWindow::on_data_received: data type "
+		<< data.get_data_type() << " in format " << data.get_format() << endl;
+
+	if (droppedUris.empty() == false)
 	{
-		// Enable all buttons
-		enableButtons = true;
+		IndexInterface *pIndex = m_settings.getIndex("MERGED");
+		set<string> locationsToIndex;
+		bool isIndexing = true;
+
+		for (list<ustring>::iterator uriIter = droppedUris.begin();
+			uriIter != droppedUris.end(); ++uriIter)
+		{
+			string uri(*uriIter);
+
+			cout << "mainWindow::on_data_received: received URI \""
+				<< uri << "\"" << endl; 
+
+			// Query the merged index
+			if (pIndex != NULL)
+			{
+				// Has this been indexed ?
+				unsigned int docId = pIndex->hasDocument(uri);
+				if (docId == 0)
+				{
+					locationsToIndex.insert(uri);
+				}
+			}
+
+			m_expandLocations.insert(uri);
+			goodDrop = true;
+		}
+
+		if (pIndex != NULL)
+		{
+			delete pIndex;
+		}
+
+		if (goodDrop == true)
+		{
+			for (set<string>::iterator locationIter = locationsToIndex.begin();
+				locationIter != locationsToIndex.end(); ++locationIter)
+			{
+				DocumentInfo docInfo("", *locationIter, "", "");
+
+				// Add this to My Web Pages
+				m_state.queue_index(docInfo);
+			}
+
+			if (locationsToIndex.empty() == true)
+			{
+				// We can run a catch-all query and expand on these documents right away
+				expand_locations();
+			}
+			// Else, do it when indexing is completed
+		}
+	}
+	else if (droppedText.empty() == false)
+	{
+		QueryProperties queryProps("", from_utf8(droppedText));
+		edit_query(queryProps, true);
+
+		cout << "mainWindow::on_data_received: received text \""
+			<< droppedText << "\"" << endl;
+
+		goodDrop = true;
 	}
 
-	removeQueryButton->set_sensitive(enableButtons);
-	queryHistoryButton->set_sensitive(enableButtons);
-	findQueryButton->set_sensitive(enableButtons);
+	context->drag_finish(goodDrop, false, time);
 }
 
 //
@@ -649,6 +735,26 @@ void mainWindow::on_enginesTreeviewSelection_changed()
 	removeIndexButton->set_sensitive(enableRemoveIndex);
 }
 
+//
+// Query list selection changed
+//
+void mainWindow::on_queryTreeviewSelection_changed()
+{
+	bool enableButtons = false;
+
+	TreeModel::iterator iter = queryTreeview->get_selection()->get_selected();
+	// Anything selected ?
+	if (iter)
+	{
+		// Enable all buttons
+		enableButtons = true;
+	}
+
+	removeQueryButton->set_sensitive(enableButtons);
+	queryHistoryButton->set_sensitive(enableButtons);
+	findQueryButton->set_sensitive(enableButtons);
+}
+
 //
 // Results tree selection changed
 //
@@ -832,7 +938,7 @@ void mainWindow::on_index_changed(ustring indexName)
 		// Connect to the edit document signal
 		pResultsTree->getDoubleClickSignal().connect(
 			sigc::mem_fun(*this, &mainWindow::on_showfromindex_activate));
-		// Connect to the label changed signal
+		// Connect to the query changed signal
 		pIndexPage->getQueryChangedSignal().connect(
 			sigc::mem_fun(*this, &mainWindow::on_query_changed));
 		// ...and to the buttons clicked signals
@@ -1006,9 +1112,6 @@ void mainWindow::on_query_changed(ustring indexName, ustring queryName)
 		return;
 	}
 
-#ifdef DEBUG
-	cout << "mainWindow::on_query_changed: called on " << queryName << endl;
-#endif
 	browse_index(indexName, queryName, 0);
 }
 
@@ -1427,6 +1530,30 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 			return;
 		}
 
+		// Refresh index lists if necessary, as they may be filtered by labels
+		if (pLabelUpdateThread->modifiedDocsIndex() == true)
+		{
+			IndexPage *pIndexPage = dynamic_cast<IndexPage*>(get_page(_("My Web Pages"), NotebookPageBox::INDEX_PAGE));
+			if (pIndexPage != NULL)
+			{
+				browse_index(_("My Web Pages"), pIndexPage->getQueryName(), 0, false);
+#ifdef DEBUG
+				cout << "mainWindow::on_thread_end: refreshed My Web Pages" << endl;
+#endif
+			}
+		}
+		if (pLabelUpdateThread->modifiedDaemonIndex() == true)
+		{
+			IndexPage *pIndexPage = dynamic_cast<IndexPage*>(get_page(_("My Documents"), NotebookPageBox::INDEX_PAGE));
+			if (pIndexPage != NULL)
+			{
+				browse_index(_("My Documents"), pIndexPage->getQueryName(), 0, false);
+#ifdef DEBUG
+				cout << "mainWindow::on_thread_end: refreshed My Documents" << endl;
+#endif
+			}
+		}
+
 		status = _("Updated label(s)");
 		set_status(status);
 	}
@@ -1618,7 +1745,10 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 	delete pThread;;
 
 	// We might be able to run a queued action
-	m_state.pop_queue(indexedUrl);
+	if (m_state.pop_queue(indexedUrl) == false)
+	{
+		expand_locations();
+	}
 
 	// Any threads left to return ?
 	if (m_state.get_threads_count() == 0)
@@ -1792,9 +1922,7 @@ void mainWindow::on_copy_activate()
 				ResultsTree *pResultsTree = pResultsPage->getTree();
 				if (pResultsTree != NULL)
 				{
-					TreeView *pTree = pResultsTree->getExtractTree();
-					if ((pTree != NULL) &&
-						(pTree->is_focus() == true))
+					if (pResultsTree->focusOnExtract() == true)
 					{
 						// Retrieve the extract
 						text = pResultsTree->getExtract();
@@ -1888,8 +2016,7 @@ void mainWindow::on_paste_activate()
 		cout << "mainWindow::on_paste_activate: query tree" << endl;
 #endif
 		// Use whatever text is in the clipboard as query name
-		// FIXME: look for \n as query fields separators ?
-		QueryProperties queryProps = QueryProperties(from_utf8(clipText), from_utf8(clipText));
+		QueryProperties queryProps("", from_utf8(clipText));
 		edit_query(queryProps, true);
 	}
 	else
@@ -2317,7 +2444,7 @@ void mainWindow::on_showfromindex_activate()
 		return;
 	}
 
-	// Update all documents
+	// Update documents properties
 	for (vector<DocumentInfo>::iterator docIter = documentsList.begin();
 		docIter != documentsList.end(); ++docIter)
 	{
@@ -2333,7 +2460,7 @@ void mainWindow::on_showfromindex_activate()
 		docIds.insert(docId);
 	}
 
-	// Now apply these labels to all the documents
+	// Apply labels
 	if (propertiesBox.changedLabels() == true)
 	{
 		LabelUpdateThread *pThread = NULL;
@@ -2353,13 +2480,6 @@ void mainWindow::on_showfromindex_activate()
 		{
 			start_thread(pThread);
 		}
-
-		// Is the index list filtered with a query ?
-		if (queryName.empty() == false)
-		{
-			// The query may filter on a label, so refresh the list
-			browse_index(indexName, queryName, 0, 0);
-		}
 	}
 }
 
@@ -2607,11 +2727,13 @@ void mainWindow::on_liveQueryEntry_changed()
 		term = liveQuery.substr(pos + 1);
 	}
 
-	// If characters are being deleted or if the term is too short, don't
-	// attempt to offer suggestions
+	// If characters are being deleted, if the term is too short or
+	// if it's a filter or a range, don't offer suggestions
 	if ((m_state.m_liveQueryLength > liveQueryLength) ||
 		(term.empty() == true) ||
-		(m_refLiveQueryCompletion->get_minimum_key_length() > term.length()))
+		(m_refLiveQueryCompletion->get_minimum_key_length() > term.length()) ||
+		(term.find(':') != string::npos) ||
+		(term.find("..") != string::npos))
 	{
 		m_state.m_liveQueryLength = liveQueryLength;
 		return;
@@ -2671,7 +2793,7 @@ void mainWindow::on_addQueryButton_clicked()
 {
 	// Even though live queries terms are now ANDed together,
 	// use them as OR terms when creating a new stored query
-	QueryProperties queryProps = QueryProperties("", liveQueryEntry->get_text());
+	QueryProperties queryProps("", liveQueryEntry->get_text());
 
 	edit_query(queryProps, true);
 }
@@ -2690,9 +2812,10 @@ void mainWindow::on_removeQueryButton_clicked()
 
 		if (m_settings.removeQuery(queryName) == true)
 		{
+			QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
+
 			// Remove records from QueryHistory
-			QueryHistory history(m_settings.getHistoryDatabaseName());
-			history.deleteItems(queryName, true);
+			queryHistory.deleteItems(queryName, true);
 
 			// Select another row
 			queryTreeview->get_selection()->unselect(iter);
@@ -2718,13 +2841,8 @@ void mainWindow::on_removeQueryButton_clicked()
 					if (pIndexPage != NULL)
 					{
 						// Synchronize the queries list with the new list 
+						// This will trigger a list refresh
 						pIndexPage->populateQueryCombobox("");
-
-						// The current query was removed, refresh the list
-						if (queryName == pIndexPage->getQueryName())
-						{
-							browse_index(pIndexPage->getTitle(), "", 0, false);
-						}
 					}
 				}
 			}
@@ -2743,13 +2861,13 @@ void mainWindow::on_queryHistoryButton_clicked()
 	// Any query selected ?
 	if (queryIter)
 	{
-		QueryHistory history(m_settings.getHistoryDatabaseName());
+		QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 		TreeModel::Row queryRow = *queryIter;
 		set<string> engineNames;
 		QueryProperties queryProps = queryRow[m_queryColumns.m_properties];
 		ustring queryName(queryRow[m_queryColumns.m_name]);
 
-		if ((history.getEngines(queryName, engineNames) == false) ||
+		if ((queryHistory.getEngines(queryName, engineNames) == false) ||
 			(engineNames.empty() == true))
 		{
 			ustring statusText = _("No history for");
@@ -2852,7 +2970,7 @@ bool mainWindow::on_queryTreeview_button_press_event(GdkEventButton *ev)
 #endif
 
 			// Edit this query's properties
-			QueryProperties queryProps = row[m_queryColumns.m_properties];
+			QueryProperties queryProps(row[m_queryColumns.m_properties]);
 			edit_query(queryProps, false);
 		}
 	}
@@ -3070,10 +3188,10 @@ void mainWindow::edit_query(QueryProperties &queryProps, bool newQuery)
 		ustring newQueryName(queryProps.getName());
 		if (newQueryName != queryName)
 		{
-			QueryHistory history(m_settings.getHistoryDatabaseName());
+			QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 
 			// Remove records from QueryHistory
-			history.deleteItems(queryName, true);
+			queryHistory.deleteItems(queryName, true);
 		}
 
 		// Update the query properties
@@ -3123,13 +3241,8 @@ void mainWindow::edit_query(QueryProperties &queryProps, bool newQuery)
 				if (pIndexPage != NULL)
 				{
 					// Synchronize the queries list with the new list
+					// This will trigger a list refresh
 					pIndexPage->populateQueryCombobox(pIndexPage->getQueryName());
-
-					// The current query may have changed, refresh the list
-					if (queryName == pIndexPage->getQueryName())
-					{
-						browse_index(pIndexPage->getTitle(), queryName, 0, false);
-					}
 				}
 			}
 		}
@@ -3273,6 +3386,9 @@ void mainWindow::run_search(const QueryProperties &queryProps)
 void mainWindow::browse_index(const ustring &indexName, const ustring &queryName,
 	unsigned int startDoc, bool changePage)
 {
+#ifdef DEBUG
+	cout << "mainWindow::browse_index: called on " << indexName << ", " << queryName << endl;
+#endif
 	// Rudimentary lock
 	if (m_state.m_browsingIndex == true)
 	{
@@ -3592,6 +3708,31 @@ bool mainWindow::start_thread(WorkerThread *pNewThread, bool inBackground)
 	return true;
 }
 
+bool mainWindow::expand_locations(void)
+{
+	// Was an expand operation delayed by indexing ?
+	if (m_expandLocations.empty() == true)
+	{
+		return false;
+	}
+
+	Url urlObj(*m_expandLocations.begin());
+	string queryName(urlObj.getFile());
+
+	if (m_expandLocations.size() > 1)
+	{
+		queryName += "...";
+	}
+	QueryProperties queryProps(queryName, "dir:/");
+
+	// Run a catch-all query and expand on these documents
+	ExpandQueryThread *pExpandQueryThread = new ExpandQueryThread(queryProps, m_expandLocations);
+	m_expandLocations.clear();
+	start_thread(pExpandQueryThread);
+
+	return true;
+}
+
 //
 // Sets the status bar text.
 //
