@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,8 +21,9 @@
 #include <glib.h>
 #include <iostream>
 #include <cstring>
-
+#ifndef USE_GIO
 #include "xdgmime/xdgmime.h"
+#endif
 
 #include "MIMEScanner.h"
 #include "StringManip.h"
@@ -33,6 +34,7 @@
 #define MIME_DEFAULTS_SECTION	"Default Applications"
 #define MIME_CACHE		"mimeinfo.cache"
 #define MIME_CACHE_SECTION	"MIME Cache"
+#define UNKNOWN_MIME_TYPE	"application/octet-stream";
 
 using std::cout;
 using std::endl;
@@ -44,6 +46,7 @@ using std::set;
 using std::vector;
 using std::pair;
 
+#ifndef USE_GIO
 static string getKeyValue(GKeyFile *pDesktopFile, const string &key)
 {
 	string value;
@@ -74,10 +77,17 @@ static string getKeyValue(GKeyFile *pDesktopFile, const string &key)
 
 	return value;
 }
+#endif
 
 MIMEAction::MIMEAction() :
 	m_multipleArgs(false),
-	m_localOnly(true)
+	m_localOnly(true),
+#ifdef USE_GIO
+	m_pAppInfo(NULL)
+#else
+	m_icon(""),
+	m_device("")
+#endif
 {
 }
 
@@ -85,11 +95,54 @@ MIMEAction::MIMEAction(const string &name, const string &cmdLine) :
 	m_multipleArgs(false),
 	m_localOnly(true),
 	m_name(name),
-	m_exec(cmdLine)
+	m_exec(cmdLine),
+#ifdef USE_GIO
+	m_pAppInfo(NULL)
+#else
+	m_icon(""),
+	m_device("")
+#endif
 {
+#ifdef USE_GIO
+	GError *pError = NULL;
+	m_pAppInfo = g_app_info_create_from_commandline(cmdLine.c_str(),
+		name.c_str(), G_APP_INFO_CREATE_SUPPORTS_URIS, &pError);
+	if (pError != NULL)
+	{
+		g_error_free(pError);
+	}
+#else
 	parseExec();
+#endif
 }
 
+#ifdef USE_GIO
+MIMEAction::MIMEAction(GAppInfo *pAppInfo) :
+	m_multipleArgs(false),
+	m_localOnly(true),
+	m_pAppInfo(NULL)
+{
+	if (pAppInfo != NULL)
+	{
+		const char *pInfo = g_app_info_get_name(pAppInfo);
+		if (pInfo != NULL)
+		{
+			m_name = pInfo;
+		}
+		pInfo = g_app_info_get_executable(pAppInfo);
+		if (pInfo != NULL)
+		{
+			m_exec = pInfo;
+		}
+		if (g_app_info_supports_uris(pAppInfo) == TRUE)
+		{
+			m_localOnly = false;
+		}
+
+		m_pAppInfo = g_app_info_dup(pAppInfo);
+	}
+}
+#else
 MIMEAction::MIMEAction(const string &desktopFile) :
 	m_multipleArgs(false),
 	m_localOnly(true),
@@ -98,19 +151,37 @@ MIMEAction::MIMEAction(const string &desktopFile) :
 	load();
 }
 
+#endif
+
 MIMEAction::MIMEAction(const MIMEAction &other) :
 	m_multipleArgs(other.m_multipleArgs),
 	m_localOnly(other.m_localOnly),
 	m_name(other.m_name),
 	m_location(other.m_location),
 	m_exec(other.m_exec),
+#ifdef USE_GIO
+	m_pAppInfo(NULL)
+#else
 	m_icon(other.m_icon),
 	m_device(other.m_device)
+#endif
 {
+#ifdef USE_GIO
+	if (other.m_pAppInfo != NULL)
+	{
+		m_pAppInfo = g_app_info_dup(other.m_pAppInfo);
+	}
+#endif
 }
 
 MIMEAction::~MIMEAction()
 {
+#ifdef USE_GIO
+	if (m_pAppInfo != NULL)
+	{
+		g_object_unref(m_pAppInfo);
+	}
+#endif
 }
 
 bool MIMEAction::operator<(const MIMEAction &other) const
@@ -132,13 +203,26 @@ MIMEAction &MIMEAction::operator=(const MIMEAction &other)
 		m_name = other.m_name;
 		m_location = other.m_location;
 		m_exec = other.m_exec;
+#ifdef USE_GIO
+		if (m_pAppInfo != NULL)
+		{
+			g_object_unref(m_pAppInfo);
+			m_pAppInfo = NULL;
+		}
+		if (other.m_pAppInfo != NULL)
+		{
+			m_pAppInfo = g_app_info_dup(other.m_pAppInfo);
+		}
+#else
 		m_icon = other.m_icon;
 		m_device = other.m_device;
+#endif
 	}
 
 	return *this;
 }
 
+#ifndef USE_GIO
 void MIMEAction::load(void)
 {
 	GKeyFile *pDesktopFile = g_key_file_new();
@@ -358,6 +442,7 @@ bool MIMECache::load(const list<string> &desktopFilesPaths)
 pthread_mutex_t MIMEScanner::m_xdgMutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_rwlock_t MIMEScanner::m_cachesLock = PTHREAD_RWLOCK_INITIALIZER;
 list<MIMECache> MIMEScanner::m_caches;
+#endif
 
 MIMEScanner::MIMEScanner()
 {
@@ -369,6 +454,12 @@ MIMEScanner::~MIMEScanner()
 
 bool MIMEScanner::initialize(const string &userPrefix, const string &systemPrefix)
 {
+#ifdef USE_GIO
+	// Initialize the GType system
+	g_type_init();
+
+	return true;
+#else
 	list<string> desktopFilesPaths;
 	string userDirectory(userPrefix + APPLICATIONS_DIRECTORY);
 	string systemDirectory(systemPrefix + APPLICATIONS_DIRECTORY);
@@ -423,6 +514,7 @@ bool MIMEScanner::initialize(const string &userPrefix, const string &systemPrefi
 	}
 
 	return foundActions;
+#endif
 }
 
 bool MIMEScanner::addCache(const string &file, const string &section,
@@ -430,6 +522,7 @@ bool MIMEScanner::addCache(const string &file, const string &section,
 {
 	bool addedCache = false;
 
+#ifndef USE_GIO
 	if (pthread_rwlock_wrlock(&m_cachesLock) == 0)
 	{
 		m_caches.push_back(MIMECache(file, section));
@@ -438,13 +531,16 @@ bool MIMEScanner::addCache(const string &file, const string &section,
 
 		pthread_rwlock_unlock(&m_cachesLock);
 	}
+#endif
 
 	return addedCache;
 }
 
 void MIMEScanner::shutdown(void)
 {
+#ifndef USE_GIO
 	xdg_mime_shutdown();
+#endif
 }
 
 void MIMEScanner::listConfigurationFiles(const string &prefix, set<string> &files)
@@ -458,36 +554,71 @@ void MIMEScanner::listConfigurationFiles(const string &prefix, set<string> &file
 
 string MIMEScanner::scanFileType(const string &fileName)
 {
-	const char *pType = NULL;
-
 	if (fileName.empty() == true)
 	{
 		return "";
 	}
 
 	// Does it have an obvious extension ?
+#ifdef USE_GIO
+	char *pType = g_content_type_guess(fileName.c_str(), NULL, 0, NULL);
+#else
+	const char *pType = NULL;
+
 	if (pthread_mutex_lock(&m_xdgMutex) == 0)
 	{
 		pType = xdg_mime_get_mime_type_from_file_name(fileName.c_str());
 
 		pthread_mutex_unlock(&m_xdgMutex);
 	}
+#endif
 
-	if ((pType == NULL) ||
-		(strncasecmp(pType, xdg_mime_type_unknown, strlen(pType)) == 0))
+	if (pType == NULL)
 	{
 		return "";
 	}
 
+#ifdef USE_GIO
+	if (g_content_type_is_unknown(pType) == TRUE)
+	{
+		g_free(pType);
+		return "";
+	}
+
+	// Get the corresponding MIME type
+	char *pMimeType = g_content_type_get_mime_type(pType);
+	if (pMimeType == NULL)
+	{
+		g_free(pType);
+		return "";
+	}
+
+	g_free(pType);
+	pType = pMimeType;
+#else
+	if (strncasecmp(pType, xdg_mime_type_unknown, strlen(pType)) == 0)
+	{
+		return "";
+	}
+#endif
+
+	string mimeType(pType);
+
 	// Quick and dirty fix to work-around shared-mime-info mistakenly identifying
 	// HTML files as Mozilla bookmarks
 	if ((fileName.find(".htm") != string::npos) &&
 		(strncasecmp(pType, "application/x-mozilla-bookmarks", strlen(pType)) == 0))
 	{
-		return "text/html";
+		mimeType = "text/html";
 	}
+#ifdef USE_GIO
+	g_free(pType);
+#endif
+#ifdef DEBUG
+	cout << "MIMEScanner::scanFileType: " << fileName << " " << mimeType << endl;
+#endif
 
-	return pType;
+	return mimeType;
 }
 
 /// Finds out the given file's MIME type.
@@ -500,30 +631,72 @@ string MIMEScanner::scanFile(const string &fileName)
 
 	string mimeType(scanFileType(fileName));
 
-	if (mimeType.empty() == true)
+	if (mimeType.empty() == false)
 	{
-		const char *pType = NULL;
-
-		// Have a peek at the file
-		if (pthread_mutex_lock(&m_xdgMutex) == 0)
-		{
-			pType = xdg_mime_get_mime_type_for_file(fileName.c_str(), NULL);
-
-			pthread_mutex_unlock(&m_xdgMutex);
-		}
-
-		if (pType != NULL)
-		{
-			mimeType = pType;
-		}
-		else
-		{
-			if (xdg_mime_type_unknown != NULL)
-			{
-				mimeType = xdg_mime_type_unknown;
-			}
-		}
+		return mimeType;
 	}
+
+#ifdef USE_GIO
+	string uri("file://");
+	uri += fileName;
+
+	// GFile will sniff the file if necessary
+	GFile *pFile = g_file_new_for_uri(uri.c_str());
+	if (pFile == NULL)
+	{
+		return UNKNOWN_MIME_TYPE;
+	}
+
+	GFileInfo *pFileInfo = g_file_query_info(pFile,
+		G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+		(GFileQueryInfoFlags)0, NULL, NULL);
+	if (pFileInfo == NULL)
+	{
+		g_object_unref(pFile);
+		return UNKNOWN_MIME_TYPE;
+	}
+
+	const char *pType = g_file_info_get_content_type(pFileInfo);
+	if (pType == NULL)
+	{
+		g_object_unref(pFileInfo);
+		g_object_unref(pFile);
+		return UNKNOWN_MIME_TYPE;
+	}
+
+	// Get the corresponding MIME type
+	char *pMimeType = g_content_type_get_mime_type(pType);
+	if (pMimeType == NULL)
+	{
+		g_object_unref(pFileInfo);
+		g_object_unref(pFile);
+		return UNKNOWN_MIME_TYPE;
+	}
+
+	mimeType = pMimeType;
+	g_free(pMimeType);
+	g_object_unref(pFileInfo);
+	g_object_unref(pFile);
+#else
+	const char *pType = NULL;
+
+	// Have a peek at the file
+	if (pthread_mutex_lock(&m_xdgMutex) == 0)
+	{
+		pType = xdg_mime_get_mime_type_for_file(fileName.c_str(), NULL);
+
+		pthread_mutex_unlock(&m_xdgMutex);
+	}
+
+	if (pType != NULL)
+	{
+		mimeType = pType;
+	}
+	else if (xdg_mime_type_unknown != NULL)
+	{
+		mimeType = xdg_mime_type_unknown;
+	}
+#endif
 #ifdef DEBUG
 	cout << "MIMEScanner::scanFile: " << fileName << " " << mimeType << endl;
 #endif
@@ -534,44 +707,124 @@ string MIMEScanner::scanFile(const string &fileName)
 /// Finds out the given URL's MIME type.
 string MIMEScanner::scanUrl(const Url &urlObj)
 {
-	string mimeType(scanFileType(urlObj.getFile()));
+	string mimeType;
 
-	if (mimeType.empty() == true)
+	// Is it a local file ?
+	if (urlObj.getProtocol() == "file")
 	{
-		// Is it a local file ?
-		if (urlObj.getProtocol() == "file")
-		{
-			string fileName = urlObj.getLocation();
-			fileName += "/";
-			fileName += urlObj.getFile();
+		string fileName(urlObj.getLocation());
+		fileName += "/";
+		fileName += urlObj.getFile();
 
-			mimeType = scanFile(fileName);
-		}
+		mimeType = scanFile(fileName);
+	}
+	else
+	{
+		mimeType = scanFileType(urlObj.getFile());
 	}
 
-	if (mimeType.empty() == true)
+	if (mimeType.empty() == false)
 	{
-		if (urlObj.getProtocol() == "http")
-		{
-			mimeType = "text/html";
-		}
-		else if (xdg_mime_type_unknown != NULL)
-		{
-			mimeType = xdg_mime_type_unknown;
-		}
+		return mimeType;
 	}
 
+	if (urlObj.getProtocol() == "http")
+	{
+		mimeType = "text/html";
+	}
+#ifdef USE_GIO
+	else
+	{
+		mimeType = UNKNOWN_MIME_TYPE;
+	}
+#else
+	else if (xdg_mime_type_unknown != NULL)
+	{
+		mimeType = xdg_mime_type_unknown;
+	}
+#endif
+#ifdef DEBUG
+	cout << "MIMEScanner::scanUrl: " << urlObj.getFile() << " " << mimeType << endl;
+#endif
+
+	return mimeType;
+}
+
+/// Finds out the given data buffer's MIME type.
+string MIMEScanner::scanData(const char *pData, unsigned int length)
+{
+	if ((pData == NULL) ||
+		(length == 0))
+	{
+		return "";
+	}
+
+#ifdef USE_GIO
+	char *pType = g_content_type_guess(NULL, (const guchar *)pData, (gsize)length, NULL);
+#else
+	const char *pType = NULL;
+
+	if (pthread_mutex_lock(&m_xdgMutex) == 0)
+	{
+		pType = xdg_mime_get_mime_type_for_data((const void *)pData, (size_t)length, NULL);
+
+		pthread_mutex_unlock(&m_xdgMutex);
+	}
+#endif
+
+	if (pType == NULL)
+	{
+		return UNKNOWN_MIME_TYPE;
+	}
+
+#ifdef USE_GIO
+	// Get the corresponding MIME type
+	char *pMimeType = g_content_type_get_mime_type(pType);
+	if (pMimeType == NULL)
+	{
+		g_free(pType);
+		return UNKNOWN_MIME_TYPE;
+	}
+
+	g_free(pType);
+	pType = pMimeType;
+#endif
+
+	string mimeType(pType);
+
+#ifdef USE_GIO
+	g_free(pType);
+#endif
+#ifdef DEBUG
+	cout << "MIMEScanner::scanData: " << mimeType << endl;
+#endif
+
 	return mimeType;
 }
 
 /// Gets parent MIME types.
-bool MIMEScanner::getParentTypes(const string &mimeType, set<string> &parentMimeTypes)
+bool MIMEScanner::getParentTypes(const string &mimeType,
+	const set<string> &allTypes, set<string> &parentMimeTypes)
 {
 	if (mimeType.empty() == true)
 	{
 		return false;
 	}
 
+#ifdef USE_GIO
+	for (set<string>::const_iterator typeIter = allTypes.begin(); typeIter != allTypes.end(); ++typeIter)
+	{
+		// FIXME: this function deals with content types, not MIME types !
+		// Check whether g_content_type_from_mime_type() exists
+		if (g_content_type_is_a(mimeType.c_str(), typeIter->c_str()) == TRUE)
+		{
+#ifdef DEBUG
+			cout << "MIMEScanner::getParentTypes: " << mimeType << " is a " << *typeIter << endl;
+#endif
+			parentMimeTypes.insert(*typeIter);
+		}
+	}
+#else
 	char **pParentTypes = xdg_mime_list_mime_parents(mimeType.c_str());
 	if ((pParentTypes != NULL) &&
 		(pParentTypes[0] != NULL))
@@ -585,6 +838,7 @@ bool MIMEScanner::getParentTypes(const string &mimeType, set<string> &parentMime
 
 		return true;
 	}
+#endif
 
 	return false;
 }
@@ -592,6 +846,19 @@ bool MIMEScanner::getParentTypes(const string &mimeType, set<string> &parentMime
 /// Adds a user-defined action for the given type.
 void MIMEScanner::addDefaultAction(const string &mimeType, const MIMEAction &typeAction)
 {
+#ifdef USE_GIO
+	if (typeAction.m_pAppInfo == NULL)
+	{
+		return;
+	}
+
+	GError *pError = NULL;
+	g_app_info_set_as_default_for_type(typeAction.m_pAppInfo, mimeType.c_str(), &pError);
+	if (pError != NULL)
+	{
+		g_error_free(pError);
+	}
+#else
 	// Custom actions get stored in a cache object which is not connected to a file
 	// We need to create this object the first time a custom action gets added
 	if (pthread_rwlock_wrlock(&m_cachesLock) == 0)
@@ -606,11 +873,46 @@ void MIMEScanner::addDefaultAction(const string &mimeType, const MIMEAction &typ
 
 		pthread_rwlock_unlock(&m_cachesLock);
 	}
+#endif
 }
 
 bool MIMEScanner::getDefaultActionsForType(const string &mimeType, set<string> &actionNames,
 	vector<MIMEAction> &typeActions)
 {
+#ifdef USE_GIO
+	GList *pAppInfoList = g_app_info_get_all_for_type(mimeType.c_str());
+	if (pAppInfoList == NULL)
+	{
+		return false;
+	}
+
+	typeActions.reserve(g_list_length(pAppInfoList));
+	for (GList *pList = pAppInfoList; pList != NULL; pList = pList->next)
+	{
+		if (pList->data == NULL)
+		{
+			continue;
+		}
+
+		GAppInfo *pAppInfo = G_APP_INFO(pList->data);
+		if (pAppInfo == NULL)
+		{
+			continue;
+		}
+
+		MIMEAction action(pAppInfo);
+#ifdef DEBUG
+		cout << "MIMEScanner::getDefaultActions: action " << action.m_name << endl;
+#endif
+
+		actionNames.insert(action.m_name);
+		typeActions.push_back(action);
+	}
+	g_list_foreach(pAppInfoList, (GFunc)g_object_unref, NULL);
+	g_list_free(pAppInfoList);
+
+	return true;
+#else
 	bool foundAction = false;
 
 	if (pthread_rwlock_rdlock(&m_cachesLock) != 0)
@@ -646,6 +948,7 @@ bool MIMEScanner::getDefaultActionsForType(const string &mimeType, set<string> &
 	pthread_rwlock_unlock(&m_cachesLock);
 
 	return foundAction;
+#endif
 }
 
 /// Determines the default action(s) for the given type.
@@ -659,6 +962,7 @@ bool MIMEScanner::getDefaultActions(const string &mimeType, vector<MIMEAction> &
 	cout << "MIMEScanner::getDefaultActions: searching for " << mimeType << endl;
 #endif
 	bool foundAction = getDefaultActionsForType(mimeType, actionNames, typeActions);
+#ifndef USE_GIO
 	if (foundAction == false)
 	{
 		// Is there an action for any of this type's parents ?
@@ -686,6 +990,7 @@ bool MIMEScanner::getDefaultActions(const string &mimeType, vector<MIMEAction> &
 		else cout << "MIMEScanner::getDefaultActions: " << mimeType << " has no parent types" << endl;
 #endif
 	}
+#endif
 
 	return foundAction;
 }
