@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,15 +21,17 @@
 
 #include <string>
 
+#include "Visibility.h"
+
 /// This class handles time conversions.
-class TimeConverter
+class PINOT_EXPORT TimeConverter
 {
 	public:
 		/// Converts into an RFC 822 timestamp.
 		static std::string toTimestamp(time_t aTime, bool inGMTime = false);
 
 		/// Converts from a RFC 822 timestamp.
-		static time_t fromTimestamp(const std::string &timestamp, bool inGMTime = false);
+		static time_t fromTimestamp(const std::string &timestamp);
 
 		/// Converts to a YYYYMMDD-formatted string.
 		static std::string toYYYYMMDDString(int year, int month, int day);
